# -*- coding: utf-8 -*-


import six

from ecl import exceptions
from ecl.tests.functional import base


class TestMEIC(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestMEIC, cls).setUpClass()
        meics = list(cls.conn.connectivity.meics())
        cls.one_meic = None
        if meics and len(meics) > 0:
            cls.one_meic = meics[0]

    def test_meics(self):
        meics = list(self.conn.connectivity.meics())
        self.assertGreaterEqual(len(meics), 0)

        for meic in meics:
            # print meic.to_dict()["service_type"]
            self.assertIsInstance(meic.mcic_id, six.string_types)
            self.assertIsInstance(meic.mcic_name, six.string_types)
            self.assertIsInstance(meic.mcic_status, six.string_types)
            self.assertIsInstance(meic.tenant_id, six.string_types)
            self.assertIsInstance(meic.tenant_name, six.string_types)
            self.assertEqual(meic.service_type, "NGC-EC")
            if meic.ngc:
                self.assertIsInstance(meic.ngc, dict)
            if meic.ec:
                self.assertIsInstance(meic.ec, dict)
            self.assertIsInstance(meic.timezone, six.string_types)

    def test_get_meic_by_id(self):
        meic = self.conn.connectivity.get_meic(self.one_meic.mcic_id)
        self.assertEqual(meic.mcic_id, self.one_meic.mcic_id)
        self.assertEqual(meic.mcic_name, self.one_meic.mcic_name)
        self.assertEqual(meic.mcic_status, self.one_meic.mcic_status)
        self.assertEqual(meic.tenant_id, self.one_meic.tenant_id)
        self.assertEqual(meic.tenant_name, self.one_meic.tenant_name)
        self.assertEqual(meic.service_id, self.one_meic.service_id)
        self.assertEqual(meic.service_type, self.one_meic.service_type)
        self.assertEqual(meic.ngc, self.one_meic.ngc)
        self.assertEqual(meic.ec, self.one_meic.ec)
        self.assertEqual(meic.timezone, self.one_meic.timezone)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.image import image_service
from ecl import resource2

class ImageCopy(resource2.Resource):
    service = image_service.ImageService()
    base_path ='/' + service.version + '/extension/image_replicator/jobs'

    # Capabilities
    allow_create = True

    # Properties
    #: An identifier job's id for the image copying.
    job_id = resource2.Body('job_id')
    #: source image_id
    source_image_id = resource2.Body('source_image_id')
    #: source region_id
    source_region_id = resource2.Body('source_region_id')
    #: source tenant_id
    source_tenant_id = resource2.Body('source_tenant_id')
    #: destination image_id
    destination_image_id = resource2.Body('destination_image_id')
    #: destination region_id
    destination_region_id = resource2.Body('destination_region_id')
    #: destination tenant_id
    destination_tenant_id = resource2.Body('destination_tenant_id')
    #: job status
    status = resource2.Body('status')
    #: progress of the image copying
    copy_progress = resource2.Body('copy_progress')

    def copy_image(self, session, image_id, tenant_id_dst):
        """Copy image to a specified region."""

        uri = self.base_path
        resp = session.post(
            uri,
            headers={"Content-Type": "application/json"},
            endpoint_filter = self.service,
            json={
                "image_id":image_id,
                "tenant_id_dst":tenant_id_dst
            }
        )
        self._translate_response(resp, has_body=True)
        return self

    def cancel_copy_image(self, session, job_id):
        """Cancel a specified image copy job."""

        uri = self.base_path + '/' + str(job_id)
        resp = session.delete(uri, endpoint_filter = self.service)
        self._translate_response(resp, has_body=False)
        return self

    def list_image_copy_jobs(self, session):
        """Lists details for image copy jobs."""

        uri = self.base_path
        resp = session.get(uri, endpoint_filter = self.service)
        resp = resp.json()
        for data in resp:
            value = self.existing(**data)
            yield value

    def get_image_copy_job(self, session, job_id):
        """Get details for a specified image copy job."""

        uri = self.base_path + '/' + str(job_id)
        resp = session.get(uri, endpoint_filter = self.service)
        self._translate_response(resp, has_body=True)
        return self



# -*- coding: utf-8 -*-

import base
from ecl.network import network_service
from ecl import resource2


class ColocationSpace(base.NetworkBaseResource):
    resource_key = 'colocation_space'
    resources_key = 'colocation_spaces'
    service = network_service.NetworkService("v2.0")
    base_path = '/' + service.version + '/colocation_spaces'

    # capabilities
    allow_get = True
    allow_list = True

    # query mappings
    _query_mapping = resource2.QueryParameters('description', 'id', 'name',
                                               "sort_key", "sort_dir",)

    # properties
    #: Colocation Space description.
    description = resource2.Body('description')
    #: Colocation Space unique id.
    id = resource2.Body('id')
    #: Colocation Space name.
    name = resource2.Body('name')



# -*- coding: utf-8 -*-

from ecl.provider_connectivity import provider_connectivity_service
from ecl.provider_connectivity.v2.base import ProviderConnectivityBaseResource
from ecl import resource2


class TenantConnection(ProviderConnectivityBaseResource):
    resources_key = "tenant_connections"
    resource_key = "tenant_connection"
    service = provider_connectivity_service.ProviderConnectivityService("v2.0")
    base_path = '/' + service.version + '/tenant_connections'

    # Capabilities
    allow_list = True
    allow_get = True
    allow_create = True
    allow_update = True
    allow_delete = True

    # query mappings
    _query_mapping = resource2.QueryParameters(
        'tenant_connection_id',
        'tenant_connection_request_id',
        'status',
        'name',
        'tenant_id',
        'name_other',
        'tenant_id_other',
        'network_id',
        'device_type',
        'device_id',
        'device_interface_id',
        'port_id'
    )

    #: tenant_connection unique ID.
    id = resource2.Body("id")

    #: tenant_connection unique ID.
    tenant_connection_id = resource2.Body("id")

    #: tenant_connection request ID.
    tenant_connection_request_id = resource2.Body(
        "tenant_connection_request_id")

    #: Name of tenant_connection.
    name = resource2.Body("name")

    #: Description
    description = resource2.Body("description")

    #: Tags
    tags = resource2.Body("tags")

    #: tenant_id
    tenant_id = resource2.Body("tenant_id")

    #: Name of network owner tenant
    name_other = resource2.Body("name_other")

    #: Description of network owner tenant
    description_other = resource2.Body("description_other")

    #: Tags of network owner tenant
    tags_other = resource2.Body("tags_other")

    #: Network owner tenant ID
    tenant_id_other = resource2.Body("tenant_id_other")

    #: Network ID
    network_id = resource2.Body("network_id")

    #: Device type, baremetal or virutal server
    device_type = resource2.Body("device_type")

    #: Device ID
    device_id = resource2.Body("device_id")

    #: Device interface ID
    device_interface_id = resource2.Body("device_interface_id")

    #: Attachment options of the connection
    attachment_opts = resource2.Body("attachment_opts")

    #: Port ID of the connection
    port_id = resource2.Body("port_id")

    #: Status of tenant_connection
    status = resource2.Body("status")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.baremetal.v2 import uefi

EXAMPLE = {
    "flavor_name": "General Purpose 2",
    "flavor_id": "fcec5541-81c3-4963-ba6f-dc24773ebf7f",
    "updated": "2012-09-07T16:56:37Z",
    "status": "SUCCESS",
    "message": "",
    "setting": {
        "hoge": {
            "value": "Enabled",
            "default": "Enabled",
            "type": "enum",
            "selection": ["Enabled", "Disabled"]
        },
        "fuga": {
            "value": "Disabled",
            "default": "Enabled",
            "type": "enum",
            "selection": ["Enabled", "Disabled"]
        }
    }
}


class TestUEFI(testtools.TestCase):

    def test_basic(self):
        sot = uefi.UEFI()
        self.assertEqual('uefi', sot.resource_key)
        self.assertEqual('uefi', sot.resources_key)
        self.assertEqual('/servers/%(server_id)s/uefi', sot.base_path)
        self.assertEqual('baremetal-server', sot.service.service_type)
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_list)

    def test_make_it(self):
        sot = uefi.UEFI(**EXAMPLE)
        self.assertEqual(EXAMPLE['flavor_name'], sot.flavor_name)
        self.assertEqual(EXAMPLE['flavor_id'], sot.flavor_id)
        self.assertEqual(EXAMPLE['updated'], sot.updated)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['message'], sot.message)
        self.assertEqual(EXAMPLE['setting'], sot.setting)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.image import image_service
from ecl import resource2

class Scheme(resource2.Resource):
    service = image_service.ImageService()
    base_path = '/' + service.version + '/schemas'

    # Properties
    #: Image's additionalProperties.
    additionalProperties = resource2.Body('additionalProperties')
    #: Image's links.
    links = resource2.Body('links')
    #: Image's name.
    name = resource2.Body('name')
    #: Properties for image.
    #: Properties list:
    #: architecture: Operating system architecture as specified in URL.
    #: status: The image status.
    #: container_format: The container format of image.
    #: min_ram: The minimum amount of RAM in MB that is required to boot the image..
    #: updated_at: The date and time when the resource was updated.The date and time stamp format is ISO 8601.
    #: owner: The ID of the owner, or tenant, of the image.
    #: min_disk: The minimum disk size in GB that is required to boot the image.
    #: tags: A list of Image tag.
    #: visibility: The image visibility.
    #: id: The unique ID for the image.
    #: size: The size of the image data, in bytes.
    #: virtual_size: The virtual size of the image.
    #: name: The name of the image.
    #: checksum: Hash of the image data used. The Image service uses this value for verification.
    #: created_at: The date and time when the resource was created.The date and time stamp format is ISO 8601.
    #: disk_format: The disk format of the image.
    #: properties: Properties, if any, that are associated with the image.
    #: protected: Indicates whether the image can be deleted.
    #: location: URL to access the image file kept in external store.
    #: self: URL for the virtual machine image.
    #: schema: URL for schema of the virtual machine image/member.
    #: direct_url: URL to access the image file kept in external store.
    #: instance_uuid: ID of instance used to create this image.
    #: kernel_id: ID of image stored in Glance that should be used as the kernel when booting an AMI-style image.
    #: os_distro: Common name of operating system distribution as specified in URL.
    #: os_version: Operating system version as specified by the distributor.
    #: ramdisk_id: ID of image stored in Glance that should be used as the ramdisk when booting an AMI-style image.
    #: member_id: Image member ID. For example, the tenant ID of the user with whom the image is being shared.
    properties = resource2.Body('properties')
    #: Metadata's definition.
    definitions = resource2.Body('definitions')

    def get_image_schema(self, session):
        """Gets a JSON schema document that represents an image
        entity."""

        uri = self.base_path + '/image'
        resp = session.get(
            uri,
            headers={"Accept": "application/json"},
            endpoint_filter=self.service
        )
        self._translate_response(resp, has_body=True)
        return self

    def get_images_schema(self, session):
        """Gets a JSON schema document that represents an images
        entity."""

        uri = self.base_path + '/images'
        resp = session.get(
            uri,
            headers={"Accept": "application/json"},
            endpoint_filter=self.service
        )
        self._translate_response(resp, has_body=True)
        return self

    def get_image_member_schema(self, session):
        """Gets a JSON schema document that represents an image member
        entity."""

        uri = self.base_path + '/member'
        resp = session.get(
            uri,
            headers={"Accept": "application/json"},
            endpoint_filter=self.service
        )
        self._translate_response(resp, has_body=True)
        return self

    def get_metadata_namespace_schema(self, session):
        """Gets a JSON schema document that represents a metadata
        definition namespace entity."""

        uri = self.base_path + '/metadefs/namespace'
        resp = session.get(
            uri,
            headers={"Accept": "application/json"},
            endpoint_filter=self.service
        )
        self._translate_response(resp, has_body=True)
        return self

    def get_metadata_object_schema(self, session):
        """Gets a JSON schema document that represents a metadata
        definition object entity."""

        uri = self.base_path + '/metadefs/object'
        resp = session.get(
            uri,
            headers={"Accept": "application/json"},
            endpoint_filter=self.service
        )
        self._translate_response(resp, has_body=True)
        return self

    def get_metadata_property_schema(self, session):
        """Gets a JSON schema document that represents a metadata
        definition property entity."""

        uri = self.base_path + '/metadefs/property'
        resp = session.get(
            uri,
            headers={"Accept": "application/json"},
            endpoint_filter=self.service
        )
        self._translate_response(resp, has_body=True)
        return self

    def get_metadata_resource_type_schema(self, session):
        """Gets a JSON schema document that represents a metadata definition namespace resource type association entity."""

        uri = self.base_path + '/metadefs/resource_type'
        resp = session.get(
            uri,
            headers={"Accept": "application/json"},
            endpoint_filter=self.service
        )
        self._translate_response(resp, has_body=True)
        return self


# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
:class:`~ecl.profile.Profile` is the class that is used to
define the various preferences for different services.  The preferences that
are currently supported are service name, region, version and interface.
The :class:`~ecl.profile.Profile` and the
:class:`~ecl.connection.Connection` classes are the most important
user facing classes.

Examples
--------


The :class:`~ecl.profile.Profile` class is constructed
with no arguments.

Set Methods
~~~~~~~~~~~

A user's preferences are set based on the service type.  Service type would
normally be something like 'compute', 'identity', 'object-store', etc.::

    from ecl import profile
    prof = profile.Profile()
    prof.set_name('compute', 'matrix')
    prof.set_region(prof.ALL, 'zion')
    prof.set_version('identity', 'v3')
    prof.set_interface('object-store', 'internal')
    for service in prof.get_services():
        print(prof.get_filter(service.service_type)

The resulting preference print out would look something like::

    service_type=compute,region=zion,service_name=matrix
    service_type=network,region=zion
    service_type=database,region=zion
    service_type=image,region=zion
    service_type=metering,region=zion
    service_type=orchestration,region=zion
    service_type=object-store,interface=internal,region=zion
    service_type=identity,region=zion,version=v3
"""

import copy
import logging
import six

from ecl.baremetal import baremetal_service
from ecl.block_store import block_store_service
from ecl.compute import compute_service
from ecl.connectivity import connectivity_service
from ecl.dedicated_hypervisor import dedicated_hypervisor_service
from ecl import exceptions
from ecl.identity import identity_service
from ecl.image import image_service
from ecl import module_loader
from ecl.network import network_service
from ecl.object_store import object_store_service
from ecl.orchestration import orchestration_service
from ecl.provider_connectivity import provider_connectivity_service
from ecl.rca import rca_service
from ecl.storage import storage_service
from ecl.security_order import security_order_service
from ecl.security_portal import security_portal_service
from ecl.sss import sss_service
from ecl.telemetry import telemetry_service
from ecl.database import database_service
from ecl.dns import dns_service
from ecl.virtual_network_appliance import virtual_network_appliance_service


_logger = logging.getLogger(__name__)
_logger.addHandler(logging.StreamHandler())


class Profile(object):

    ALL = "*"
    """Wildcard service identifier representing all services."""

    def __init__(self, plugins=None):
        """User preference for each service.

        :param list plugins: List of entry point namespaces to load.

        Create a new :class:`~ecl.profile.Profile`
        object with no preferences defined, but knowledge of the services.
        Services are identified by their service type, e.g.: 'identity',
        'compute', etc.
        """
        self._services = {}
        self._add_service(compute_service.ComputeService(version="v2"))
        self._add_service(
            connectivity_service.ConnectivityService(version="v1"))
        self._add_service(identity_service.IdentityService(version="v3"))
        self._add_service(image_service.ImageService(version="v2"))
        self._add_service(network_service.NetworkService(version="v2"))
        self._add_service(sss_service.SssService(version="v1"))
        self._add_service(
            object_store_service.ObjectStoreService(version="v1"))
        self._add_service(
            orchestration_service.OrchestrationService(version="v1"))
        self._add_service(
            provider_connectivity_service.ProviderConnectivityService(
                version="v2"))
        self._add_service(telemetry_service.TelemetryService(version="v2"))
        self._add_service(block_store_service.BlockStoreService(version="v2"))
        self._add_service(storage_service.StorageService(version="v1"))
        self._add_service(
            security_order_service.SecurityOrderService(version="v1"))
        self._add_service(
            security_portal_service.SecurityPortalService(version="v1"))
        self._add_service(rca_service.RcaService(version="v1"))
        self._add_service(baremetal_service.BaremetalService(version="v2"))
        self._add_service(
            dedicated_hypervisor_service.DedicatedHypervisorService(
                version="v1"))
        self._add_service(database_service.DatabaseService(version="v1"))
        self._add_service(dns_service.DnsService(version="v2"))
        self._add_service(
            virtual_network_appliance_service.VirtualNetworkApplianceService(
                version="v1"))

        # NOTE: The Metric service is not added here as it currently
        # only retrieves the /capabilities API.

        if plugins:
            for plugin in plugins:
                self._load_plugin(plugin)
        self.service_keys = sorted(self._services.keys())

    def __repr__(self):
        return repr(self._services)

    def _add_service(self, serv):
        serv.interface = None
        self._services[serv.service_type] = serv

    def _load_plugin(self, namespace):
        """Load a service plugin.

        :param str namespace: Entry point namespace
        """
        services = module_loader.load_service_plugins(namespace)
        for service_type in services:
            if service_type in self._services:
                _logger.debug("Overriding %s with %s", service_type,
                              services[service_type])
            self._add_service(services[service_type])

    def get_filter(self, service):
        """Get a service preference.

        :param str service: Desired service type.
        """
        return copy.copy(self._get_filter(service))

    def _get_filter(self, service):
        """Get a service preference.

        :param str service: Desired service type.
        """
        serv = self._services.get(service, None)
        if serv is not None:
            return serv
        msg = ("Service %s not in list of valid services: %s" %
               (service, self.service_keys))
        raise exceptions.SDKException(msg)

    def _get_services(self, service):
        return self.service_keys if service == self.ALL else [service]

    def _setter(self, service, attr, value):
        for service in self._get_services(service):
            setattr(self._get_filter(service), attr, value)

    def get_services(self):
        """Get a list of all the known services."""
        services = []
        for name, service in six.iteritems(self._services):
            services.append(service)
        return services

    def set_name(self, service, name):
        """Set the desired name for the specified service.

        :param str service: Service type.
        :param str name: Desired service name.
        """
        self._setter(service, "service_name", name)

    def set_region(self, service, region):
        """Set the desired region for the specified service.

        :param str service: Service type.
        :param str region: Desired service region.
        """
        self._setter(service, "region", region)

    def set_version(self, service, version):
        """Set the desired version for the specified service.

        :param str service: Service type.
        :param str version: Desired service version.
        """
        self._get_filter(service).version = version

    def set_api_version(self, service, api_version):
        """Set the desired API micro-version for the specified service.

        :param str service: Service type.
        :param str api_version: Desired service API micro-version.
        """
        self._setter(service, "api_version", api_version)

    def set_interface(self, service, interface):
        """Set the desired interface for the specified service.

        :param str service: Service type.
        :param str interface: Desired service interface.
        """
        self._setter(service, "interface", interface)



"""
Exception definitions for Management service(SSS).
"""
import json
from ecl import exceptions


class HttpException(exceptions.HttpException):
    pass


class NotFoundException(HttpException):
    pass



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.image import image_service


class TestImageService(testtools.TestCase):

    def test_service(self):
        sot = image_service.ImageService()
        self.assertEqual('image', sot.service_type)
        self.assertEqual('public', sot.interface)
        self.assertIsNone(sot.region)
        self.assertIsNone(sot.service_name)
        self.assertEqual(1, len(sot.valid_versions))
        self.assertEqual('v2', sot.valid_versions[0].module)
        self.assertEqual('v2', sot.valid_versions[0].path)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import testtools

from keystoneauth1 import exceptions as _exceptions

from ecl import exceptions
from ecl import profile
from ecl import session


class TestSession(testtools.TestCase):

    def test_init_user_agent_none(self):
        sot = session.Session(None)
        self.assertTrue(sot.user_agent.startswith("eclsdk"))

    def test_init_user_agent_set(self):
        sot = session.Session(None, user_agent="testing/123")
        self.assertTrue(sot.user_agent.startswith("testing/123 eclsdk"))

    def test_init_with_single_api_request(self):
        prof = profile.Profile()
        prof.set_api_version('storage', '1.2')

        sot = session.Session(prof)

        # The assertion acutally tests the property assigned in parent class
        self.assertEqual({'ecl-api-version': 'storage 1.2'},
                         sot.additional_headers)

    def test_init_with_multi_api_requests(self):
        prof = profile.Profile()
        prof.set_api_version('storage', '1.2')
        prof.set_api_version('compute', '2.15')

        sot = session.Session(prof)

        versions = sot.additional_headers['ecl-api-version']
        requests = [req.strip() for req in versions.split(',')]
        self.assertIn('storage 1.2', requests)
        self.assertIn('compute 2.15', requests)

    def test_init_with_no_api_requests(self):
        prof = profile.Profile()

        sot = session.Session(prof)

        self.assertEqual({}, sot.additional_headers)

    def test_map_exceptions_not_found_exception(self):
        ksa_exc = _exceptions.HttpError(message="test", http_status=404)
        func = mock.Mock(side_effect=ksa_exc)

        os_exc = self.assertRaises(
            exceptions.NotFoundException, session.map_exceptions(func))
        self.assertIsInstance(os_exc, exceptions.NotFoundException)
        self.assertEqual(ksa_exc.message, os_exc.message)
        self.assertEqual(ksa_exc.http_status, os_exc.http_status)
        self.assertEqual(ksa_exc, os_exc.cause)

    def test_map_exceptions_http_exception(self):
        ksa_exc = _exceptions.HttpError(message="test", http_status=400)
        func = mock.Mock(side_effect=ksa_exc)

        os_exc = self.assertRaises(
            exceptions.HttpException, session.map_exceptions(func))
        self.assertIsInstance(os_exc, exceptions.HttpException)
        self.assertEqual(ksa_exc.message, os_exc.message)
        self.assertEqual(ksa_exc.http_status, os_exc.http_status)
        self.assertEqual(ksa_exc, os_exc.cause)

    def test_map_exceptions_sdk_exception_1(self):
        ksa_exc = _exceptions.ClientException()
        func = mock.Mock(side_effect=ksa_exc)

        os_exc = self.assertRaises(
            exceptions.SDKException, session.map_exceptions(func))
        self.assertIsInstance(os_exc, exceptions.SDKException)
        self.assertEqual(ksa_exc, os_exc.cause)

    def test_map_exceptions_sdk_exception_2(self):
        ksa_exc = _exceptions.VersionNotAvailable()
        func = mock.Mock(side_effect=ksa_exc)

        os_exc = self.assertRaises(
            exceptions.SDKException, session.map_exceptions(func))
        self.assertIsInstance(os_exc, exceptions.SDKException)
        self.assertEqual(ksa_exc, os_exc.cause)

    def _test__get_endpoint_versions(self, body, versions):
        sot = session.Session(None)

        fake_response = mock.Mock()
        fake_response.json = mock.Mock(return_value=body)
        sot.get = mock.Mock(return_value=fake_response)

        scheme = "https"
        netloc = "devstack"
        root = scheme + "://" + netloc

        rv = sot._get_endpoint_versions(
            "compute", "%s://%s/v2.1/projectidblahblah" % (scheme, netloc))

        sot.get.assert_called_with(root)

        self.assertEqual(rv[0], root)
        self.assertEqual(rv[1], versions)

    def test__get_endpoint_versions_nested(self):
        versions = [{"id": "v2.0"}, {"id": "v2.1"}]
        body = {"versions": {"values": versions}}
        self._test__get_endpoint_versions(body, versions)

    def test__get_endpoint_versions(self):
        versions = [{"id": "v2.0"}, {"id": "v2.1"}]
        body = {"versions": versions}
        self._test__get_endpoint_versions(body, versions)

    def test__get_endpoint_versions_exception(self):
        sot = session.Session(None)

        fake_response = mock.Mock()
        fake_response.json = mock.Mock(return_value={})
        sot.get = mock.Mock(return_value=fake_response)

        self.assertRaises(exceptions.EndpointNotFound,
                          sot._get_endpoint_versions, "service", "endpoint")

    def test__parse_version(self):
        sot = session.Session(None)

        self.assertEqual(sot._parse_version("2"), (2, -1))
        self.assertEqual(sot._parse_version("v2"), (2, -1))
        self.assertEqual(sot._parse_version("v2.1"), (2, 1))
        self.assertRaises(ValueError, sot._parse_version, "lol")

    def test__get_version_match_none(self):
        sot = session.Session(None)

        self.assertRaises(
            exceptions.EndpointNotFound,
            sot._get_version_match, [], None, "service", "root", False)

    def test__get_version_match_fuzzy(self):
        match = "http://devstack/v2.1/"
        versions = [{"id": "v2.0",
                     "links": [{"href": "http://devstack/v2/",
                                "rel": "self"}]},
                    {"id": "v2.1",
                     "links": [{"href": match,
                                "rel": "self"}]}]

        sot = session.Session(None)
        # Look for a v2 match, which we internally denote as a minor
        # version of -1 so we can find the highest matching minor.
        rv = sot._get_version_match(versions, session.Version(2, -1),
                                    "service", "root", False)
        self.assertEqual(rv, match)

    def test__get_version_match_exact(self):
        match = "http://devstack/v2/"
        versions = [{"id": "v2.0",
                     "links": [{"href": match,
                                "rel": "self"}]},
                    {"id": "v2.1",
                     "links": [{"href": "http://devstack/v2.1/",
                                "rel": "self"}]}]

        sot = session.Session(None)
        rv = sot._get_version_match(versions, session.Version(2, 0),
                                    "service", "root", False)
        self.assertEqual(rv, match)

    def test__get_version_match_fragment(self):
        root = "http://cloud.net"
        match = "/v2/"
        versions = [{"id": "v2.0", "links": [{"href": match, "rel": "self"}]}]

        sot = session.Session(None)
        rv = sot._get_version_match(versions, session.Version(2, 0),
                                    "service", root, False)
        self.assertEqual(rv, root+match)

    def test__get_version_match_project_id(self):
        match = "http://devstack/v2/"
        project_id = "asdf123"
        versions = [{"id": "v2.0", "links": [{"href": match, "rel": "self"}]}]

        sot = session.Session(None)
        sot.get_project_id = mock.Mock(return_value=project_id)
        rv = sot._get_version_match(versions, session.Version(2, 0),
                                    "service", "root", True)
        self.assertEqual(rv, match + project_id)

    def test_get_endpoint_cached(self):
        sot = session.Session(None)
        service_type = "compute"
        interface = "public"
        endpoint = "the world wide web"

        sot.endpoint_cache[(service_type, interface)] = endpoint
        rv = sot.get_endpoint(service_type=service_type, interface=interface)
        self.assertEqual(rv, endpoint)



# -*- coding: utf-8 -*-

from ecl.block_store import block_store_service
from ecl import resource


class Limit(resource.Resource):
    resource_key = "limits"
    resources_key = "limits"
    base_path = "/limits"
    service = block_store_service.BlockStoreService()

    # capabilities
    allow_retrieve = True

    # Properties
    #: A rate representing this Limit.
    rate = resource.prop("rate")
    #: A list of absolute props of the Limit.
    absolute = resource.prop("absolute")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import six

from ecl import exceptions
from ecl.orchestration.v1 import _proxy
from ecl.orchestration.v1 import resource
from ecl.orchestration.v1 import software_config as sc
from ecl.orchestration.v1 import software_deployment as sd
from ecl.orchestration.v1 import stack
from ecl.orchestration.v1 import template
from ecl.tests.unit import test_proxy_base2


class TestOrchestrationProxy(test_proxy_base2.TestProxyBase):
    def setUp(self):
        super(TestOrchestrationProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)

    def test_create_stack(self):
        self.verify_create(self.proxy.create_stack, stack.Stack)

    def test_create_stack_preview(self):
        method_kwargs = {"preview": True, "x": 1, "y": 2, "z": 3}
        self.verify_create(self.proxy.create_stack, stack.StackPreview,
                           method_kwargs=method_kwargs)

    def test_find_stack(self):
        self.verify_find(self.proxy.find_stack, stack.Stack)

    def test_stacks(self):
        self.verify_list(self.proxy.stacks, stack.Stack, paginated=False)

    def test_get_stack(self):
        self.verify_get(self.proxy.get_stack, stack.Stack)

    def test_update_stack(self):
        self.verify_update(self.proxy.update_stack, stack.Stack)

    def test_delete_stack(self):
        self.verify_delete(self.proxy.delete_stack, stack.Stack, False)

    def test_delete_stack_ignore(self):
        self.verify_delete(self.proxy.delete_stack, stack.Stack, True)

    @mock.patch.object(stack.Stack, 'check')
    def test_check_stack_with_stack_object(self, mock_check):
        stk = stack.Stack(id='FAKE_ID')

        res = self.proxy.check_stack(stk)

        self.assertIsNone(res)
        mock_check.assert_called_once_with(self.proxy.session)

    @mock.patch.object(stack.Stack, 'existing')
    def test_check_stack_with_stack_ID(self, mock_stack):
        stk = mock.Mock()
        mock_stack.return_value = stk

        res = self.proxy.check_stack('FAKE_ID')

        self.assertIsNone(res)
        mock_stack.assert_called_once_with(id='FAKE_ID')
        stk.check.assert_called_once_with(self.proxy.session)

    @mock.patch.object(stack.Stack, 'find')
    def test_resources_with_stack_object(self, mock_find):
        stack_id = '1234'
        stack_name = 'test_stack'
        stk = stack.Stack(id=stack_id, name=stack_name)

        self.verify_list(self.proxy.resources, resource.Resource,
                         paginated=False, method_args=[stk],
                         expected_kwargs={'stack_name': stack_name,
                                          'stack_id': stack_id})

        self.assertEqual(0, mock_find.call_count)

    @mock.patch.object(stack.Stack, 'find')
    def test_resources_with_stack_name(self, mock_find):
        stack_id = '1234'
        stack_name = 'test_stack'
        stk = stack.Stack(id=stack_id, name=stack_name)
        mock_find.return_value = stk

        self.verify_list(self.proxy.resources, resource.Resource,
                         paginated=False, method_args=[stack_id],
                         expected_kwargs={'stack_name': stack_name,
                                          'stack_id': stack_id})

        mock_find.assert_called_once_with(mock.ANY, stack_id,
                                          ignore_missing=True)

    @mock.patch.object(stack.Stack, 'find')
    @mock.patch.object(resource.Resource, 'list')
    def test_resources_stack_not_found(self, mock_list, mock_find):
        stack_name = 'test_stack'
        mock_find.side_effect = exceptions.ResourceNotFound(
            'No stack found for test_stack')

        ex = self.assertRaises(exceptions.ResourceNotFound,
                               self.proxy.resources, stack_name)
        self.assertEqual('ResourceNotFound: No stack found for test_stack',
                         six.text_type(ex))

    def test_create_software_config(self):
        self.verify_create(self.proxy.create_software_config,
                           sc.SoftwareConfig)

    def test_software_configs(self):
        self.verify_list(self.proxy.software_configs, sc.SoftwareConfig,
                         paginated=True)

    def test_get_software_config(self):
        self.verify_get(self.proxy.get_software_config, sc.SoftwareConfig)

    def test_delete_software_config(self):
        self.verify_delete(self.proxy.delete_software_config,
                           sc.SoftwareConfig, True)
        self.verify_delete(self.proxy.delete_software_config,
                           sc.SoftwareConfig, False)

    def test_create_software_deployment(self):
        self.verify_create(self.proxy.create_software_deployment,
                           sd.SoftwareDeployment)

    def test_software_deployments(self):
        self.verify_list(self.proxy.software_deployments,
                         sd.SoftwareDeployment, paginated=False)

    def test_get_software_deployment(self):
        self.verify_get(self.proxy.get_software_deployment,
                        sd.SoftwareDeployment)

    def test_update_software_deployment(self):
        self.verify_update(self.proxy.update_software_deployment,
                           sd.SoftwareDeployment)

    def test_delete_software_deployment(self):
        self.verify_delete(self.proxy.delete_software_deployment,
                           sd.SoftwareDeployment, True)
        self.verify_delete(self.proxy.delete_software_deployment,
                           sd.SoftwareDeployment, False)

    @mock.patch.object(template.Template, 'validate')
    def test_validate_template(self, mock_validate):
        tmpl = mock.Mock()
        env = mock.Mock()
        tmpl_url = 'A_URI'
        ignore_errors = 'a_string'

        res = self.proxy.validate_template(tmpl, env, tmpl_url, ignore_errors)

        mock_validate.assert_called_once_with(
            self.proxy.session, tmpl, environment=env, template_url=tmpl_url,
            ignore_errors=ignore_errors)
        self.assertEqual(mock_validate.return_value, res)

    def test_validate_template_invalid_request(self):
        err = self.assertRaises(exceptions.InvalidRequest,
                                self.proxy.validate_template,
                                None, template_url=None)
        self.assertEqual("'template_url' must be specified when template is "
                         "None", six.text_type(err))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.database import database_service
from ecl import resource2
from ecl import exceptions


class User(resource2.Resource):
    resource_key = "user"
    resources_key = "users"
    base_path = '/instances/%(instance_id)s/users'
    service = database_service.DatabaseService()

    # _query_mapping = resource2.QueryParameters()

    # Capabilities
    allow_create = True
    allow_delete = True
    allow_list = True
    allow_update = True
    allow_get = True

    # Properties
    #: User's name of Database instance.
    name = resource2.Body('name', alternate_id=True)

    #: Allowed access host of user.
    host = resource2.Body('host')

    #: Relevant database of this user.
    databases = resource2.Body('databases', type=list)

    #: User's password of Database instance.
    #: This parameter is only used in instance creation.
    password = resource2.Body('password')

    #: ID of instance associated with this user
    instance_id = resource2.URI('instance_id')

    def create(self, session, instance_id, **attrs):
        base = self.base_path % {"instance_id": instance_id}
        body = {"users": [attrs]}
        resp = session.post(base, endpoint_filter=self.service, json=body,
                            headers={"Accept": "application/json"})
        self._translate_response(resp, has_body=False)
        return self

    def grant(self, session, instance_id, user, databases):
        base = self.base_path % {"instance_id": instance_id}
        uri = "%s/%s/databases" % (base, user)
        resp = session.put(uri, endpoint_filter=self.service,
                           json={"databases": databases})
        self._translate_response(resp, has_body=False)
        return self

    def revoke(self, session, instance_id, user, database):
        base = self.base_path % {"instance_id": instance_id}
        uri = uri = "%s/%s/databases/%s" % (base, user, database)
        resp = session.delete(uri, endpoint_filter=self.service)
        self._translate_response(resp, has_body=False)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import itertools

import mock
import six

from ecl import exceptions
from ecl import format
from ecl import resource2
from ecl import session
from ecl.tests.unit import base


class TestComponent(base.TestCase):

    class ExampleComponent(resource2._BaseComponent):
        key = "_example"

    # Since we're testing ExampleComponent, which is as isolated as we
    # can test _BaseComponent due to it's needing to be a data member
    # of a class that has an attribute on the parent class named `key`,
    # each test has to implement a class with a name that is the same
    # as ExampleComponent.key, which should be a dict containing the
    # keys and values to test against.

    def test_implementations(self):
        self.assertEqual("_body", resource2.Body.key)
        self.assertEqual("_header", resource2.Header.key)
        self.assertEqual("_uri", resource2.URI.key)

    def test_creation(self):
        sot = resource2._BaseComponent("name", type=int, default=1,
                                       alternate_id=True)

        self.assertEqual("name", sot.name)
        self.assertEqual(int, sot.type)
        self.assertEqual(1, sot.default)
        self.assertTrue(sot.alternate_id)

    def test_get_no_instance(self):
        sot = resource2._BaseComponent("test")

        # Test that we short-circuit everything when given no instance.
        result = sot.__get__(None, None)
        self.assertIsNone(result)

    # NOTE: Some tests will use a default=1 setting when testing result
    # values that should be None because the default-for-default is also None.
    def test_get_name_None(self):
        name = "name"

        class Parent(object):
            _example = {name: None}

        instance = Parent()
        sot = TestComponent.ExampleComponent(name, default=1)

        # Test that we short-circuit any typing of a None value.
        result = sot.__get__(instance, None)
        self.assertIsNone(result)

    def test_get_default(self):
        expected_result = 123

        class Parent(object):
            _example = {}

        instance = Parent()
        # NOTE: type=dict but the default value is an int. If we didn't
        # short-circuit the typing part of __get__ it would fail.
        sot = TestComponent.ExampleComponent("name", type=dict,
                                             default=expected_result)

        # Test that we directly return any default value.
        result = sot.__get__(instance, None)
        self.assertEqual(expected_result, result)

    def test_get_name_untyped(self):
        name = "name"
        expected_result = 123

        class Parent(object):
            _example = {name: expected_result}

        instance = Parent()
        sot = TestComponent.ExampleComponent("name")

        # Test that we return any the value as it is set.
        result = sot.__get__(instance, None)
        self.assertEqual(expected_result, result)

    # The code path for typing after a raw value has been found is the same.
    def test_get_name_typed(self):
        name = "name"
        value = "123"

        class Parent(object):
            _example = {name: value}

        instance = Parent()
        sot = TestComponent.ExampleComponent("name", type=int)

        # Test that we run the underlying value through type conversion.
        result = sot.__get__(instance, None)
        self.assertEqual(int(value), result)

    def test_get_name_formatter(self):
        name = "name"
        value = "123"
        expected_result = "one hundred twenty three"

        class Parent(object):
            _example = {name: value}

        class FakeFormatter(object):
            @classmethod
            def deserialize(cls, value):
                return expected_result

        instance = Parent()
        sot = TestComponent.ExampleComponent("name", type=FakeFormatter)

        # Mock out issubclass rather than having an actual format.Formatter
        # This can't be mocked via decorator, isolate it to wrapping the call.
        mock_issubclass = mock.Mock(return_value=True)
        module = six.moves.builtins.__name__
        with mock.patch("%s.issubclass" % module, mock_issubclass):
            result = sot.__get__(instance, None)
        self.assertEqual(expected_result, result)

    def test_set_name_untyped(self):
        name = "name"
        expected_value = "123"

        class Parent(object):
            _example = {}

        instance = Parent()
        sot = TestComponent.ExampleComponent("name")

        # Test that we don't run the value through type conversion.
        sot.__set__(instance, expected_value)
        self.assertEqual(expected_value, instance._example[name])

    def test_set_name_typed(self):
        expected_value = "123"

        class Parent(object):
            _example = {}

        instance = Parent()

        # The type we give to ExampleComponent has to be an actual type,
        # not an instance, so we can't get the niceties of a mock.Mock
        # instance that would allow us to call `assert_called_once_with` to
        # ensure that we're sending the value through the type.
        # Instead, we use this tiny version of a similar thing.
        class FakeType(object):
            calls = []

            def __init__(self, arg):
                FakeType.calls.append(arg)

        sot = TestComponent.ExampleComponent("name", type=FakeType)

        # Test that we run the value through type conversion.
        sot.__set__(instance, expected_value)
        self.assertEqual([expected_value], FakeType.calls)

    def test_set_name_formatter(self):
        expected_value = "123"

        class Parent(object):
            _example = {}

        instance = Parent()

        # As with test_set_name_typed, create a pseudo-Mock to track what
        # gets called on the type.
        class FakeFormatter(format.Formatter):
            calls = []

            @classmethod
            def serialize(cls, arg):
                FakeFormatter.calls.append(arg)

        sot = TestComponent.ExampleComponent("name", type=FakeFormatter)

        # Test that we run the value through type conversion.
        sot.__set__(instance, expected_value)
        self.assertEqual([expected_value], FakeFormatter.calls)

    def test_delete_name(self):
        name = "name"
        expected_value = "123"

        class Parent(object):
            _example = {name: expected_value}

        instance = Parent()

        sot = TestComponent.ExampleComponent("name")

        sot.__delete__(instance)

        self.assertNotIn(name, instance._example)

    def test_delete_name_doesnt_exist(self):
        name = "name"
        expected_value = "123"

        class Parent(object):
            _example = {"what": expected_value}

        instance = Parent()

        sot = TestComponent.ExampleComponent(name)

        sot.__delete__(instance)

        self.assertNotIn(name, instance._example)


class TestComponentManager(base.TestCase):

    def test_create_basic(self):
        sot = resource2._ComponentManager()
        self.assertEqual(dict(), sot.attributes)
        self.assertEqual(set(), sot._dirty)

    def test_create_unsynced(self):
        attrs = {"hey": 1, "hi": 2, "hello": 3}
        sync = False

        sot = resource2._ComponentManager(attributes=attrs, synchronized=sync)
        self.assertEqual(attrs, sot.attributes)
        self.assertEqual(set(attrs.keys()), sot._dirty)

    def test_create_synced(self):
        attrs = {"hey": 1, "hi": 2, "hello": 3}
        sync = True

        sot = resource2._ComponentManager(attributes=attrs, synchronized=sync)
        self.assertEqual(attrs, sot.attributes)
        self.assertEqual(set(), sot._dirty)

    def test_getitem(self):
        key = "key"
        value = "value"
        attrs = {key: value}

        sot = resource2._ComponentManager(attributes=attrs)
        self.assertEqual(value, sot.__getitem__(key))

    def test_setitem_new(self):
        key = "key"
        value = "value"

        sot = resource2._ComponentManager()
        sot.__setitem__(key, value)

        self.assertIn(key, sot.attributes)
        self.assertIn(key, sot.dirty)

    def test_setitem_unchanged(self):
        key = "key"
        value = "value"
        attrs = {key: value}

        sot = resource2._ComponentManager(attributes=attrs, synchronized=True)
        # This shouldn't end up in the dirty list since we're just re-setting.
        sot.__setitem__(key, value)

        self.assertEqual(value, sot.attributes[key])
        self.assertNotIn(key, sot.dirty)

    def test_delitem(self):
        key = "key"
        value = "value"
        attrs = {key: value}

        sot = resource2._ComponentManager(attributes=attrs, synchronized=True)
        sot.__delitem__(key)

        self.assertIsNone(sot.dirty[key])

    def test_iter(self):
        attrs = {"key": "value"}
        sot = resource2._ComponentManager(attributes=attrs)
        self.assertItemsEqual(iter(attrs), sot.__iter__())

    def test_len(self):
        attrs = {"key": "value"}
        sot = resource2._ComponentManager(attributes=attrs)
        self.assertEqual(len(attrs), sot.__len__())

    def test_dirty(self):
        key = "key"
        key2 = "key2"
        value = "value"
        attrs = {key: value}
        sot = resource2._ComponentManager(attributes=attrs, synchronized=False)
        self.assertEqual({key: value}, sot.dirty)

        sot.__setitem__(key2, value)
        self.assertEqual({key: value, key2: value}, sot.dirty)

    def test_clean(self):
        key = "key"
        value = "value"
        attrs = {key: value}
        sot = resource2._ComponentManager(attributes=attrs, synchronized=False)
        self.assertEqual(attrs, sot.dirty)

        sot.clean()

        self.assertEqual(dict(), sot.dirty)


class Test_Request(base.TestCase):

    def test_create(self):
        uri = 1
        body = 2
        headers = 3

        sot = resource2._Request(uri, body, headers)

        self.assertEqual(uri, sot.uri)
        self.assertEqual(body, sot.body)
        self.assertEqual(headers, sot.headers)


class TestQueryParameters(base.TestCase):

    def test_create(self):
        location = "location"
        mapping = {"first_name": "first-name"}

        sot = resource2.QueryParameters(location, **mapping)

        self.assertEqual({"location": "location", "first_name": "first-name"},
                         sot._mapping)

    def test_transpose_unmapped(self):
        location = "location"
        mapping = {"first_name": "first-name"}

        sot = resource2.QueryParameters(location, **mapping)
        result = sot._transpose({"location": "Brooklyn",
                                 "first_name": "Brian",
                                 "last_name": "Curtin"})

        # last_name isn't mapped and shouldn't be included
        self.assertEqual({"location": "Brooklyn", "first-name": "Brian"},
                         result)

    def test_transpose_not_in_query(self):
        location = "location"
        mapping = {"first_name": "first-name"}

        sot = resource2.QueryParameters(location, **mapping)
        result = sot._transpose({"location": "Brooklyn"})

        # first_name not being in the query shouldn't affect results
        self.assertEqual({"location": "Brooklyn"},
                         result)


class TestResource(base.TestCase):

    def test_initialize_basic(self):
        body = {"body": 1}
        header = {"header": 2}
        uri = {"uri": 3}
        everything = dict(itertools.chain(body.items(), header.items(),
                                          uri.items()))

        mock_collect = mock.Mock()
        mock_collect.return_value = body, header, uri

        with mock.patch.object(resource2.Resource,
                               "_collect_attrs", mock_collect):
            sot = resource2.Resource(synchronized=False, **everything)
            mock_collect.assert_called_once_with(everything)

        self.assertIsInstance(sot._body, resource2._ComponentManager)
        self.assertEqual(body, sot._body.dirty)
        self.assertIsInstance(sot._header, resource2._ComponentManager)
        self.assertEqual(header, sot._header.dirty)
        self.assertIsInstance(sot._uri, resource2._ComponentManager)
        self.assertEqual(uri, sot._uri.dirty)

        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_list)
        self.assertFalse(sot.allow_head)
        self.assertFalse(sot.patch_update)
        self.assertFalse(sot.put_create)

    def test_repr(self):
        a = {"a": 1}
        b = {"b": 2}
        c = {"c": 3}

        class Test(resource2.Resource):
            def __init__(self):
                self._body = mock.Mock()
                self._body.attributes.items = mock.Mock(
                    return_value=a.items())

                self._header = mock.Mock()
                self._header.attributes.items = mock.Mock(
                    return_value=b.items())

                self._uri = mock.Mock()
                self._uri.attributes.items = mock.Mock(
                    return_value=c.items())

        the_repr = repr(Test())

        # Don't test the arguments all together since the dictionary order
        # they're rendered in can't be depended on, nor does it matter.
        self.assertIn("ecl.tests.unit.test_resource2.Test", the_repr)
        self.assertIn("a=1", the_repr)
        self.assertIn("b=2", the_repr)
        self.assertIn("c=3", the_repr)

    def test_equality(self):
        class Example(resource2.Resource):
            x = resource2.Body("x")
            y = resource2.Header("y")
            z = resource2.URI("z")

        e1 = Example(x=1, y=2, z=3)
        e2 = Example(x=1, y=2, z=3)
        e3 = Example(x=0, y=0, z=0)

        self.assertEqual(e1, e2)
        self.assertNotEqual(e1, e3)

    def test__update(self):
        sot = resource2.Resource()

        body = "body"
        header = "header"
        uri = "uri"

        sot._collect_attrs = mock.Mock(return_value=(body, header, uri))
        sot._body.update = mock.Mock()
        sot._header.update = mock.Mock()
        sot._uri.update = mock.Mock()

        args = {"arg": 1}
        sot._update(**args)

        sot._collect_attrs.assert_called_once_with(args)
        sot._body.update.assert_called_once_with(body)
        sot._header.update.assert_called_once_with(header)
        sot._uri.update.assert_called_once_with(uri)

    def test__collect_attrs(self):
        sot = resource2.Resource()

        expected_attrs = ["body", "header", "uri"]

        sot._consume_attrs = mock.Mock()
        sot._consume_attrs.side_effect = expected_attrs

        # It'll get passed an empty dict at the least.
        actual_attrs = sot._collect_attrs(dict())

        self.assertItemsEqual(expected_attrs, actual_attrs)

    def test__consume_attrs(self):
        serverside_key1 = "someKey1"
        clientside_key1 = "some_key1"
        serverside_key2 = "someKey2"
        clientside_key2 = "some_key2"
        value1 = "value1"
        value2 = "value2"
        mapping = {clientside_key1: serverside_key1,
                   clientside_key2: serverside_key2}

        other_key = "otherKey"
        other_value = "other"
        attrs = {clientside_key1: value1,
                 serverside_key2: value2,
                 other_key: other_value}

        sot = resource2.Resource()

        result = sot._consume_attrs(mapping, attrs)

        # Make sure that the expected key was consumed and we're only
        # left with the other stuff.
        self.assertDictEqual({other_key: other_value}, attrs)

        # Make sure that after we've popped our relevant client-side
        # key off that we are returning it keyed off of its server-side
        # name.
        self.assertDictEqual({serverside_key1: value1,
                              serverside_key2: value2}, result)

    def test__mapping_defaults(self):
        # Check that even on an empty class, we get the expected
        # built-in attributes.

        self.assertIn("location", resource2.Resource._header_mapping())
        self.assertIn("name", resource2.Resource._body_mapping())
        self.assertIn("id", resource2.Resource._body_mapping())

    def test__mapping_overrides(self):
        # Iterating through the MRO used to wipe out overrides of mappings
        # found in base classes.
        new_name = "MyName"
        new_id = "MyID"

        class Test(resource2.Resource):
            name = resource2.Body(new_name)
            id = resource2.Body(new_id)

        mapping = Test._body_mapping()

        self.assertEqual(new_name, mapping["name"])
        self.assertEqual(new_id, mapping["id"])

    def test__body_mapping(self):
        class Test(resource2.Resource):
            x = resource2.Body("x")
            y = resource2.Body("y")
            z = resource2.Body("z")

        self.assertIn("x", Test._body_mapping())
        self.assertIn("y", Test._body_mapping())
        self.assertIn("z", Test._body_mapping())

    def test__header_mapping(self):
        class Test(resource2.Resource):
            x = resource2.Header("x")
            y = resource2.Header("y")
            z = resource2.Header("z")

        self.assertIn("x", Test._header_mapping())
        self.assertIn("y", Test._header_mapping())
        self.assertIn("z", Test._header_mapping())

    def test__uri_mapping(self):
        class Test(resource2.Resource):
            x = resource2.URI("x")
            y = resource2.URI("y")
            z = resource2.URI("z")

        self.assertIn("x", Test._uri_mapping())
        self.assertIn("y", Test._uri_mapping())
        self.assertIn("z", Test._uri_mapping())

    def test__alternate_id_None(self):
        self.assertEqual("", resource2.Resource._alternate_id())

    def test__alternate_id(self):
        class Test(resource2.Resource):
            alt = resource2.Body("the_alt", alternate_id=True)

        self.assertTrue("the_alt", Test._alternate_id())

        value1 = "lol"
        sot = Test(alt=value1)
        self.assertEqual(sot.alt, value1)
        self.assertEqual(sot.id, value1)

        value2 = "rofl"
        sot = Test(the_alt=value2)
        self.assertEqual(sot.alt, value2)
        self.assertEqual(sot.id, value2)

    def test__get_id_instance(self):
        class Test(resource2.Resource):
            id = resource2.Body("id")

        value = "id"
        sot = Test(id=value)

        self.assertEqual(value, sot._get_id(sot))

    def test__get_id_instance_alternate(self):
        class Test(resource2.Resource):
            attr = resource2.Body("attr", alternate_id=True)

        value = "id"
        sot = Test(attr=value)

        self.assertEqual(value, sot._get_id(sot))

    def test__get_id_value(self):
        value = "id"
        self.assertEqual(value, resource2.Resource._get_id(value))

    def test_to_dict(self):

        class Test(resource2.Resource):
            foo = resource2.Header('foo')
            bar = resource2.Body('bar')

        res = Test(id='FAKE_ID')

        expected = {
            'id': 'FAKE_ID',
            'name': None,
            'location': None,
            'foo': None,
            'bar': None
        }
        self.assertEqual(expected, res.to_dict())

    def test_to_dict_no_body(self):

        class Test(resource2.Resource):
            foo = resource2.Header('foo')
            bar = resource2.Body('bar')

        res = Test(id='FAKE_ID')

        expected = {
            'location': None,
            'foo': None,
        }
        self.assertEqual(expected, res.to_dict(body=False))

    def test_to_dict_no_header(self):

        class Test(resource2.Resource):
            foo = resource2.Header('foo')
            bar = resource2.Body('bar')

        res = Test(id='FAKE_ID')

        expected = {
            'id': 'FAKE_ID',
            'name': None,
            'bar': None
        }
        self.assertEqual(expected, res.to_dict(headers=False))

    def test_to_dict_ignore_none(self):

        class Test(resource2.Resource):
            foo = resource2.Header('foo')
            bar = resource2.Body('bar')

        res = Test(id='FAKE_ID', bar='BAR')

        expected = {
            'id': 'FAKE_ID',
            'bar': 'BAR',
        }
        self.assertEqual(expected, res.to_dict(ignore_none=True))

    def test_to_dict_with_mro(self):

        class Parent(resource2.Resource):
            foo = resource2.Header('foo')
            bar = resource2.Body('bar')

        class Child(Parent):
            foo_new = resource2.Header('foo_baz_server')
            bar_new = resource2.Body('bar_baz_server')

        res = Child(id='FAKE_ID')

        expected = {
            'foo': None,
            'bar': None,
            'foo_new': None,
            'bar_new': None,
            'id': 'FAKE_ID',
            'location': None,
            'name': None
        }
        self.assertEqual(expected, res.to_dict())

    def test_to_dict_value_error(self):

        class Test(resource2.Resource):
            foo = resource2.Header('foo')
            bar = resource2.Body('bar')

        res = Test(id='FAKE_ID')

        err = self.assertRaises(ValueError,
                                res.to_dict, body=False, headers=False)
        self.assertEqual('At least one of `body` or `headers` must be True',
                         six.text_type(err))

    def test_to_dict_with_mro_no_override(self):

        class Parent(resource2.Resource):
            header = resource2.Header('HEADER')
            body = resource2.Body('BODY')

        class Child(Parent):
            # The following two properties are not supposed to be overridden
            # by the parent class property values.
            header = resource2.Header('ANOTHER_HEADER')
            body = resource2.Body('ANOTHER_BODY')

        res = Child(id='FAKE_ID', body='BODY_VALUE', header='HEADER_VALUE')

        expected = {
            'body': 'BODY_VALUE',
            'header': 'HEADER_VALUE',
            'id': 'FAKE_ID',
            'location': None,
            'name': None
        }
        self.assertEqual(expected, res.to_dict())

    def test_new(self):
        class Test(resource2.Resource):
            attr = resource2.Body("attr")

        value = "value"
        sot = Test.new(attr=value)

        self.assertIn("attr", sot._body.dirty)
        self.assertEqual(value, sot.attr)

    def test_existing(self):
        class Test(resource2.Resource):
            attr = resource2.Body("attr")

        value = "value"
        sot = Test.existing(attr=value)

        self.assertNotIn("attr", sot._body.dirty)
        self.assertEqual(value, sot.attr)

    def test__prepare_request_with_id(self):
        class Test(resource2.Resource):
            base_path = "/something"
            body_attr = resource2.Body("x")
            header_attr = resource2.Header("y")

        the_id = "id"
        body_value = "body"
        header_value = "header"
        sot = Test(id=the_id, body_attr=body_value, header_attr=header_value,
                   synchronized=False)

        result = sot._prepare_request(requires_id=True)

        self.assertEqual("something/id", result.uri)
        self.assertEqual({"x": body_value, "id": the_id}, result.body)
        self.assertEqual({"y": header_value}, result.headers)

    def test__prepare_request_missing_id(self):
        sot = resource2.Resource(id=None)

        self.assertRaises(exceptions.InvalidRequest,
                          sot._prepare_request, requires_id=True)

    def test__prepare_request_with_key(self):
        key = "key"

        class Test(resource2.Resource):
            base_path = "/something"
            resource_key = key
            body_attr = resource2.Body("x")
            header_attr = resource2.Header("y")

        body_value = "body"
        header_value = "header"
        sot = Test(body_attr=body_value, header_attr=header_value,
                   synchronized=False)

        result = sot._prepare_request(requires_id=False, prepend_key=True)

        self.assertEqual("/something", result.uri)
        self.assertEqual({key: {"x": body_value}}, result.body)
        self.assertEqual({"y": header_value}, result.headers)

    def test__filter_component(self):
        client_name = "client_name"
        server_name = "serverName"
        value = "value"
        # Include something in the mapping that we don't receive
        # so the branch that looks at existence in the compoment is checked.
        mapping = {client_name: server_name, "other": "blah"}
        component = {server_name: value, "something": "else"}

        sot = resource2.Resource()
        result = sot._filter_component(component, mapping)

        # The something:else mapping should not make it into here.
        self.assertEqual({server_name: value}, result)

    def test__translate_response_no_body(self):
        class Test(resource2.Resource):
            attr = resource2.Header("attr")

        response = mock.Mock()
        response.headers = dict()

        sot = Test()
        sot._filter_component = mock.Mock(return_value={"attr": "value"})

        sot._translate_response(response, has_body=False)

        self.assertEqual(dict(), sot._header.dirty)
        self.assertEqual("value", sot.attr)

    def test__translate_response_with_body_no_resource_key(self):
        class Test(resource2.Resource):
            attr = resource2.Body("attr")

        body = {"attr": "value"}
        response = mock.Mock()
        response.headers = dict()
        response.json.return_value = body

        sot = Test()
        sot._filter_component = mock.Mock(side_effect=[body, dict()])

        sot._translate_response(response, has_body=True)

        self.assertEqual("value", sot.attr)
        self.assertEqual(dict(), sot._body.dirty)
        self.assertEqual(dict(), sot._header.dirty)

    def test__translate_response_with_body_with_resource_key(self):
        key = "key"

        class Test(resource2.Resource):
            resource_key = key
            attr = resource2.Body("attr")

        body = {"attr": "value"}
        response = mock.Mock()
        response.headers = dict()
        response.json.return_value = {key: body}

        sot = Test()
        sot._filter_component = mock.Mock(side_effect=[body, dict()])

        sot._translate_response(response, has_body=True)

        self.assertEqual("value", sot.attr)
        self.assertEqual(dict(), sot._body.dirty)
        self.assertEqual(dict(), sot._header.dirty)

    def test_cant_do_anything(self):
        class Test(resource2.Resource):
            allow_create = False
            allow_get = False
            allow_update = False
            allow_delete = False
            allow_head = False
            allow_list = False

        sot = Test()

        # The first argument to all of these operations is the session,
        # but we raise before we get to it so just pass anything in.
        self.assertRaises(exceptions.MethodNotSupported, sot.create, "")
        self.assertRaises(exceptions.MethodNotSupported, sot.get, "")
        self.assertRaises(exceptions.MethodNotSupported, sot.delete, "")
        self.assertRaises(exceptions.MethodNotSupported, sot.head, "")

        # list is a generator so you need to begin consuming
        # it in order to exercise the failure.
        the_list = sot.list("")
        self.assertRaises(exceptions.MethodNotSupported, next, the_list)

        # Update checks the dirty list first before even trying to see
        # if the call can be made, so fake a dirty list.
        sot._body = mock.Mock()
        sot._body.dirty = mock.Mock(return_value={"x": "y"})
        self.assertRaises(exceptions.MethodNotSupported, sot.update, "")


class TestResourceActions(base.TestCase):

    def setUp(self):
        super(TestResourceActions, self).setUp()

        self.service_name = "service"
        self.base_path = "base_path"

        class Test(resource2.Resource):
            service = self.service_name
            base_path = self.base_path
            allow_create = True
            allow_get = True
            allow_head = True
            allow_update = True
            allow_delete = True
            allow_list = True

        self.test_class = Test

        self.request = mock.Mock(spec=resource2._Request)
        self.request.uri = "uri"
        self.request.body = "body"
        self.request.headers = "headers"

        self.response = mock.Mock()

        self.sot = Test(id="id")
        self.sot._prepare_request = mock.Mock(return_value=self.request)
        self.sot._translate_response = mock.Mock()

        self.session = mock.Mock(spec=session.Session)
        self.session.create = mock.Mock(return_value=self.response)
        self.session.get = mock.Mock(return_value=self.response)
        self.session.put = mock.Mock(return_value=self.response)
        self.session.patch = mock.Mock(return_value=self.response)
        self.session.post = mock.Mock(return_value=self.response)
        self.session.delete = mock.Mock(return_value=self.response)
        self.session.head = mock.Mock(return_value=self.response)

    def _test_create(self, cls, requires_id=False, prepend_key=False):
        id = "id" if requires_id else None
        sot = cls(id=id)
        sot._prepare_request = mock.Mock(return_value=self.request)
        sot._translate_response = mock.Mock()

        result = sot.create(self.session, prepend_key=prepend_key)

        sot._prepare_request.assert_called_once_with(
            requires_id=requires_id, prepend_key=prepend_key)
        if requires_id:
            self.session.put.assert_called_once_with(
                self.request.uri,
                endpoint_filter=self.service_name,
                json=self.request.body, headers=self.request.headers)
        else:
            self.session.post.assert_called_once_with(
                self.request.uri,
                endpoint_filter=self.service_name,
                json=self.request.body, headers=self.request.headers)

        sot._translate_response.assert_called_once_with(self.response)
        self.assertEqual(result, sot)

    def test_put_create(self):
        class Test(resource2.Resource):
            service = self.service_name
            base_path = self.base_path
            allow_create = True
            put_create = True

        self._test_create(Test, requires_id=True, prepend_key=True)

    def test_post_create(self):
        class Test(resource2.Resource):
            service = self.service_name
            base_path = self.base_path
            allow_create = True
            put_create = False

        self._test_create(Test, requires_id=False, prepend_key=True)

    def test_get(self):
        result = self.sot.get(self.session)

        self.sot._prepare_request.assert_called_once_with(requires_id=True)
        self.session.get.assert_called_once_with(
            self.request.uri, endpoint_filter=self.service_name)

        self.sot._translate_response.assert_called_once_with(self.response)
        self.assertEqual(result, self.sot)

    def test_get_not_requires_id(self):
        result = self.sot.get(self.session, False)

        self.sot._prepare_request.assert_called_once_with(requires_id=False)
        self.session.get.assert_called_once_with(
            self.request.uri, endpoint_filter=self.service_name)

        self.sot._translate_response.assert_called_once_with(self.response)
        self.assertEqual(result, self.sot)

    def test_head(self):
        result = self.sot.head(self.session)

        self.sot._prepare_request.assert_called_once_with()
        self.session.head.assert_called_once_with(
            self.request.uri,
            endpoint_filter=self.service_name,
            headers={"Accept": ""})

        self.sot._translate_response.assert_called_once_with(self.response)
        self.assertEqual(result, self.sot)

    def _test_update(self, patch_update=False, prepend_key=True,
                     has_body=True):
        self.sot.patch_update = patch_update

        # Need to make sot look dirty so we can attempt an update
        self.sot._body = mock.Mock()
        self.sot._body.dirty = mock.Mock(return_value={"x": "y"})

        self.sot.update(self.session, prepend_key=prepend_key,
                        has_body=has_body)

        self.sot._prepare_request.assert_called_once_with(
            prepend_key=prepend_key)

        if patch_update:
            self.session.patch.assert_called_once_with(
                self.request.uri,
                endpoint_filter=self.service_name,
                json=self.request.body, headers=self.request.headers)
        else:
            self.session.put.assert_called_once_with(
                self.request.uri,
                endpoint_filter=self.service_name,
                json=self.request.body, headers=self.request.headers)

        self.sot._translate_response.assert_called_once_with(
            self.response, has_body=has_body)

    def test_update_put(self):
        self._test_update(patch_update=False, prepend_key=True, has_body=True)

    def test_update_patch(self):
        self._test_update(patch_update=True, prepend_key=False, has_body=False)

    def test_update_not_dirty(self):
        self.sot._body = mock.Mock()
        self.sot._body.dirty = dict()
        self.sot._header = mock.Mock()
        self.sot._header.dirty = dict()

        self.sot.update(self.session)

        self.session.put.assert_not_called()

    def test_delete(self):
        result = self.sot.delete(self.session)

        self.sot._prepare_request.assert_called_once_with()
        self.session.delete.assert_called_once_with(
            self.request.uri,
            endpoint_filter=self.service_name,
            headers={"Accept": ""})

        self.sot._translate_response.assert_called_once_with(
            self.response, has_body=False)
        self.assertEqual(result, self.sot)

    # NOTE: As list returns a generator, testing it requires consuming
    # the generator. Wrap calls to self.sot.list in a `list`
    # and then test the results as a list of responses.
    def test_list_empty_response(self):
        mock_response = mock.Mock()
        mock_response.json.return_value = []

        self.session.get.return_value = mock_response

        result = list(self.sot.list(self.session))

        self.session.get.assert_called_once_with(
            self.base_path,
            endpoint_filter=self.service_name,
            headers={"Accept": "application/json"},
            params={})

        self.assertEqual([], result)

    def test_list_one_page_response_paginated(self):
        id_value = 1
        mock_response = mock.Mock()
        mock_response.json.side_effect = [[{"id": id_value}],
                                          []]

        self.session.get.return_value = mock_response

        # Ensure that we break out of the loop on a paginated call
        # that still only results in one page of data.
        results = list(self.sot.list(self.session, paginated=True))

        self.assertEqual(1, len(results))

        # Look at the `params` argument to each of the get calls that
        # were made.
        self.session.get.call_args_list[0][1]["params"] = {}
        self.session.get.call_args_list[1][1]["params"] = {"marker": id_value}
        self.assertEqual(id_value, results[0].id)
        self.assertIsInstance(results[0], self.test_class)

    def test_list_one_page_response_not_paginated(self):
        id_value = 1
        mock_response = mock.Mock()
        mock_response.json.return_value = [{"id": id_value}]

        self.session.get.return_value = mock_response

        results = list(self.sot.list(self.session, paginated=False))

        self.session.get.assert_called_once_with(
            self.base_path,
            endpoint_filter=self.service_name,
            headers={"Accept": "application/json"},
            params={})

        self.assertEqual(1, len(results))
        self.assertEqual(id_value, results[0].id)
        self.assertIsInstance(results[0], self.test_class)

    def test_list_one_page_response_resources_key(self):
        key = "resources"

        class Test(self.test_class):
            resources_key = key

        id_value = 1
        mock_response = mock.Mock()
        mock_response.json.return_value = {key: [{"id": id_value}]}

        self.session.get.return_value = mock_response

        sot = Test()

        results = list(sot.list(self.session))

        self.session.get.assert_called_once_with(
            self.base_path,
            endpoint_filter=self.service_name,
            headers={"Accept": "application/json"},
            params={})

        self.assertEqual(1, len(results))
        self.assertEqual(id_value, results[0].id)
        self.assertIsInstance(results[0], self.test_class)

    def test_list_multi_page_response_not_paginated(self):
        ids = [1, 2]
        mock_response = mock.Mock()
        mock_response.json.side_effect = [[{"id": ids[0]}],
                                          [{"id": ids[1]}]]

        self.session.get.return_value = mock_response

        results = list(self.sot.list(self.session, paginated=False))

        self.assertEqual(1, len(results))
        self.assertEqual(ids[0], results[0].id)
        self.assertIsInstance(results[0], self.test_class)

    def test_list_query_params(self):
        id = 1
        qp = "query param!"
        qp_name = "query-param"
        uri_param = "uri param!"

        mock_response = mock.Mock()
        mock_response.json.side_effect = [[{"id": id}],
                                          []]

        self.session.get.return_value = mock_response

        class Test(self.test_class):
            _query_mapping = resource2.QueryParameters(query_param=qp_name)
            base_path = "/%(something)s/blah"
            something = resource2.URI("something")

        results = list(Test.list(self.session, paginated=True,
                                 query_param=qp, something=uri_param))

        self.assertEqual(1, len(results))

        # Look at the `params` argument to each of the get calls that
        # were made.
        self.session.get.call_args_list[0][1]["params"] = {qp_name: qp}

        self.assertEqual(self.session.get.call_args_list[0][0][0],
                         Test.base_path % {"something": uri_param})

    def test_list_multi_page_response_paginated(self):
        # This tests our ability to stop making calls once
        # we've received all of the data. However, this tests
        # the case that we always receive full pages of data
        # and then the signal that there is no more data - an empty list.
        # In this case, we need to make one extra request beyond
        # the end of data to ensure we've received it all.
        ids = [1, 2]
        resp1 = mock.Mock()
        resp1.json.return_value = [{"id": ids[0]}]
        resp2 = mock.Mock()
        resp2.json.return_value = [{"id": ids[1]}]
        resp3 = mock.Mock()
        resp3.json.return_value = []

        self.session.get.side_effect = [resp1, resp2, resp3]

        results = self.sot.list(self.session, paginated=True)

        result0 = next(results)
        self.assertEqual(result0.id, ids[0])
        self.session.get.assert_called_with(
            self.base_path,
            endpoint_filter=self.service_name,
            headers={"Accept": "application/json"},
            params={})

        result1 = next(results)
        self.assertEqual(result1.id, ids[1])
        self.session.get.assert_called_with(
            self.base_path,
            endpoint_filter=self.service_name,
            headers={"Accept": "application/json"},
            params={"limit": 1, "marker": 1})

        self.assertRaises(StopIteration, next, results)
        self.session.get.assert_called_with(
            self.base_path,
            endpoint_filter=self.service_name,
            headers={"Accept": "application/json"},
            params={"limit": 1, "marker": 2})

    def test_list_multi_page_early_termination(self):
        # This tests our ability to be somewhat smart when evaluating
        # the contents of the responses. When we receive a full page
        # of data, we can be smart about terminating our responses
        # once we see that we've received a page with less data than
        # expected, saving one request.
        ids = [1, 2, 3]
        resp1 = mock.Mock()
        resp1.json.return_value = [{"id": ids[0]}, {"id": ids[1]}]
        resp2 = mock.Mock()
        resp2.json.return_value = [{"id": ids[2]}]

        self.session.get.side_effect = [resp1, resp2]

        results = self.sot.list(self.session, paginated=True)

        # Get the first page's two items
        result0 = next(results)
        self.assertEqual(result0.id, ids[0])
        result1 = next(results)
        self.assertEqual(result1.id, ids[1])
        self.session.get.assert_called_with(
            self.base_path,
            endpoint_filter=self.service_name,
            headers={"Accept": "application/json"},
            params={})

        # Second page only has one item
        result2 = next(results)
        self.assertEqual(result2.id, ids[2])
        self.session.get.assert_called_with(
            self.base_path,
            endpoint_filter=self.service_name,
            headers={"Accept": "application/json"},
            params={"limit": 2, "marker": 2})

        # Ensure we're done after those three items
        self.assertRaises(StopIteration, next, results)

        # Ensure we only made two calls to get this done
        self.assertEqual(2, len(self.session.get.call_args_list))


class TestResourceFind(base.TestCase):

    def setUp(self):
        super(TestResourceFind, self).setUp()

        self.result = 1

        class Base(resource2.Resource):

            @classmethod
            def existing(cls, **kwargs):
                raise exceptions.NotFoundException

            @classmethod
            def list(cls, session):
                return None

        class OneResult(Base):

            @classmethod
            def _get_one_match(cls, *args):
                return self.result

        class NoResults(Base):

            @classmethod
            def _get_one_match(cls, *args):
                return None

        self.no_results = NoResults
        self.one_result = OneResult

    def test_find_short_circuit(self):
        value = 1

        class Test(resource2.Resource):

            @classmethod
            def existing(cls, **kwargs):
                mock_match = mock.Mock()
                mock_match.get.return_value = value
                return mock_match

        result = Test.find("session", "name")

        self.assertEqual(result, value)

    def test_no_match_raise(self):
        self.assertRaises(exceptions.ResourceNotFound, self.no_results.find,
                          "session", "name", ignore_missing=True)

    def test_no_match_return(self):
        self.assertIsNone(
            self.no_results.find("session", "name", ignore_missing=True))

    def test_find_result(self):
        self.assertEqual(self.result, self.one_result.find("session", "name"))

    def test_match_empty_results(self):
        self.assertIsNone(resource2.Resource._get_one_match("name", []))

    def test_no_match_by_name(self):
        the_name = "Brian"

        match = mock.Mock(spec=resource2.Resource)
        match.name = the_name

        result = resource2.Resource._get_one_match("Richard", [match])

        self.assertIsNone(result, match)

    def test_single_match_by_name(self):
        the_name = "Brian"

        match = mock.Mock(spec=resource2.Resource)
        match.name = the_name

        result = resource2.Resource._get_one_match(the_name, [match])

        self.assertIs(result, match)

    def test_single_match_by_id(self):
        the_id = "Brian"

        match = mock.Mock(spec=resource2.Resource)
        match.id = the_id

        result = resource2.Resource._get_one_match(the_id, [match])

        self.assertIs(result, match)

    def test_single_match_by_alternate_id(self):
        the_id = "Richard"

        class Test(resource2.Resource):
            other_id = resource2.Body("other_id", alternate_id=True)

        match = Test(other_id=the_id)
        result = Test._get_one_match(the_id, [match])

        self.assertIs(result, match)

    def test_multiple_matches(self):
        the_id = "Brian"

        match = mock.Mock(spec=resource2.Resource)
        match.id = the_id

        self.assertRaises(
            exceptions.DuplicateResource,
            resource2.Resource._get_one_match, the_id, [match, match])


class TestWaitForStatus(base.TestCase):

    def test_immediate_status(self):
        status = "loling"
        resource = mock.Mock()
        resource.status = status

        result = resource2.wait_for_status("session", resource, status,
                                           "failures", "interval", "wait")

        self.assertEqual(result, resource)

    @mock.patch("time.sleep", return_value=None)
    def test_status_match(self, mock_sleep):
        status = "loling"
        resource = mock.Mock()

        # other gets past the first check, two anothers gets through
        # the sleep loop, and the third matches
        statuses = ["other", "another", "another", status]
        type(resource).status = mock.PropertyMock(side_effect=statuses)

        result = resource2.wait_for_status("session", resource, status,
                                           None, 1, 5)

        self.assertEqual(result, resource)

    @mock.patch("time.sleep", return_value=None)
    def test_status_fails(self, mock_sleep):
        status = "loling"
        failure = "crying"
        resource = mock.Mock()

        # other gets past the first check, the first failure doesn't
        # match the expected, the third matches the failure,
        # the fourth is used in creating the exception message
        statuses = ["other", failure, failure, failure]
        type(resource).status = mock.PropertyMock(side_effect=statuses)

        self.assertRaises(exceptions.ResourceFailure,
                          resource2.wait_for_status,
                          "session", resource, status, [failure], 1, 5)

    @mock.patch("time.sleep", return_value=None)
    def test_timeout(self, mock_sleep):
        status = "loling"
        resource = mock.Mock()

        # The first "other" gets past the first check, and then three
        # pairs of "other" statuses run through the sleep counter loop,
        # after which time should be up. This is because we have a
        # one second interval and three second waiting period.
        statuses = ["other"] * 7
        type(resource).status = mock.PropertyMock(side_effect=statuses)

        self.assertRaises(exceptions.ResourceTimeout,
                          resource2.wait_for_status,
                          "session", resource, status, None, 1, 3)

    def test_no_sleep(self):
        resource = mock.Mock()
        statuses = ["other"]
        type(resource).status = mock.PropertyMock(side_effect=statuses)

        self.assertRaises(exceptions.ResourceTimeout,
                          resource2.wait_for_status,
                          "session", resource, "status", None, 0, -1)


class TestWaitForDelete(base.TestCase):

    @mock.patch("time.sleep", return_value=None)
    def test_success(self, mock_sleep):
        resource = mock.Mock()
        resource.get.side_effect = [None, None, exceptions.NotFoundException]

        result = resource2.wait_for_delete("session", resource, 1, 3)

        self.assertEqual(result, resource)

    @mock.patch("time.sleep", return_value=None)
    def test_timeout(self, mock_sleep):
        resource = mock.Mock()
        resource.get.side_effect = [None, None, None]

        self.assertRaises(exceptions.ResourceTimeout,
                          resource2.wait_for_delete,
                          "session", resource, 1, 3)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl import resource
from ecl.telemetry import telemetry_service


class Statistics(resource.Resource):
    """.. caution:: This API is a work in progress and is subject to change."""
    id_attribute = 'meter_name'
    resource_key = 'statistics'
    base_path = '/meters/%(meter_name)s/statistics'
    service = telemetry_service.TelemetryService()

    # Supported Operations
    allow_list = True

    # Path Parameter
    meter_name = resource.prop('meter_name')

    # Properties
    #: The selectable aggregate value(s)
    aggregate = resource.prop('aggregate')
    #: The average of all of the volume values seen in the data
    avg = resource.prop('avg')
    #: The number of samples seen
    count = resource.prop('count')
    #: The difference, in seconds, between the oldest and newest timestamp
    duration = resource.prop('duration')
    #: UTC date and time of the oldest timestamp, or the query end time.
    duration_end_at = resource.prop('duration_end')
    #: UTC date and time of the earliest timestamp, or the query start time.
    duration_start_at = resource.prop('duration_start')
    #: Dictionary of field names for group, if groupby statistics are requested
    group_by = resource.prop('groupby')
    #: The maximum volume seen in the data
    max = resource.prop('max')
    #: The minimum volume seen in the data
    min = resource.prop('min')
    #: The difference, in seconds, between the period start and end
    period = resource.prop('period')
    #: UTC date and time of the period end.
    period_end_at = resource.prop('period_end')
    #: UTC date and time of the period start.
    period_start_at = resource.prop('period_start')
    #: The total of all of the volume values seen in the data
    sum = resource.prop('sum')
    #: The unit type of the data set
    unit = resource.prop('unit')

    @classmethod
    def list(cls, session, limit=None, marker=None, path_args=None,
             paginated=False, **params):
        url = cls._get_url(path_args)
        resp = session.get(url, endpoint_filter=cls.service, params=params)
        for stat in resp.json():
            yield cls.existing(**stat)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import testtools

from ecl.compute.v2 import limits

ABSOLUTE_LIMITS = {
    "maxImageMeta": 128,
    "maxPersonality": 5,
    "maxPersonalitySize": 10240,
    "maxSecurityGroupRules": 20,
    "maxSecurityGroups": 10,
    "maxServerMeta": 128,
    "maxTotalCores": 20,
    "maxTotalFloatingIps": 10,
    "maxTotalInstances": 10,
    "maxTotalKeypairs": 100,
    "maxTotalRAMSize": 51200,
    "maxServerGroups": 10,
    "maxServerGroupMembers": 10,
    "totalFloatingIpsUsed": 1,
    "totalSecurityGroupsUsed": 2,
    "totalRAMUsed": 4,
    "totalInstancesUsed": 5,
    "totalServerGroupsUsed": 6,
    "totalCoresUsed": 7,

    #: New missing attributes
    "totalSnapshotsUsed": 0,
    "maxTotalBackups": 0,
    "maxTotalVolumeGigabytes": 0,
    "maxTotalSnapshots": 0,
    "maxTotalBackupGigabytes": 0,
    "totalBackupGigabytesUsed": 0,
    "maxTotalVolumes": 0,
    "totalVolumesUsed": 0,
    "totalBackupsUsed": 0,
    "totalGigabytesUsed": 0,
}

RATE_LIMIT = {
    "limit": [
        {
            "next-available": "2012-11-27T17:22:18Z",
            "remaining": 120,
            "unit": "MINUTE",
            "value": 120,
            "verb": "POST"
        },
    ],
    "regex": ".*",
    "uri": "*"
}

LIMITS_BODY = {
    "limits": {
        "absolute": ABSOLUTE_LIMITS,
        "rate": [RATE_LIMIT]
    }
}


class TestAbsoluteLimits(testtools.TestCase):

    def test_basic(self):
        sot = limits.AbsoluteLimits()
        self.assertIsNone(sot.resource_key)
        self.assertIsNone(sot.resources_key)
        self.assertEqual("", sot.base_path)
        self.assertIsNone(sot.service)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_list)

    def test_make_it(self):
        sot = limits.AbsoluteLimits(**ABSOLUTE_LIMITS)
        self.assertEqual(ABSOLUTE_LIMITS["maxImageMeta"], sot.image_meta)
        self.assertEqual(ABSOLUTE_LIMITS["maxPersonality"], sot.personality)
        self.assertEqual(ABSOLUTE_LIMITS["maxPersonalitySize"],
                         sot.personality_size)
        self.assertEqual(ABSOLUTE_LIMITS["maxSecurityGroupRules"],
                         sot.security_group_rules)
        self.assertEqual(ABSOLUTE_LIMITS["maxSecurityGroups"],
                         sot.security_groups)
        self.assertEqual(ABSOLUTE_LIMITS["maxServerMeta"], sot.server_meta)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalCores"], sot.cores)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalFloatingIps"],
                         sot.floating_ips)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalInstances"],
                         sot.instances)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalKeypairs"],
                         sot.keypairs)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalRAMSize"],
                         sot.ram)
        self.assertEqual(ABSOLUTE_LIMITS["maxServerGroups"], sot.server_groups)
        self.assertEqual(ABSOLUTE_LIMITS["maxServerGroupMembers"],
                         sot.server_group_members)
        self.assertEqual(ABSOLUTE_LIMITS["totalFloatingIpsUsed"],
                         sot.floating_ips_used)
        self.assertEqual(ABSOLUTE_LIMITS["totalSecurityGroupsUsed"],
                         sot.security_groups_used)
        self.assertEqual(ABSOLUTE_LIMITS["totalRAMUsed"], sot.ram_used)
        self.assertEqual(ABSOLUTE_LIMITS["totalInstancesUsed"],
                         sot.instances_used)
        self.assertEqual(ABSOLUTE_LIMITS["totalServerGroupsUsed"],
                         sot.server_groups_used)
        self.assertEqual(ABSOLUTE_LIMITS["totalCoresUsed"],
                         sot.cores_used)

        #: new missing attributes

        self.assertEqual(ABSOLUTE_LIMITS["totalSnapshotsUsed"],
                         sot.snapshots_used)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalBackups"],
                         sot.backups)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalVolumeGigabytes"],
                         sot.volume_gigabytes)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalSnapshots"],
                         sot.snapshots)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalBackupGigabytes"],
                         sot.backup_gigabytes)
        self.assertEqual(ABSOLUTE_LIMITS["totalBackupGigabytesUsed"],
                         sot.backup_gigabytes_used)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalVolumes"],
                         sot.volumes)
        self.assertEqual(ABSOLUTE_LIMITS["totalVolumesUsed"],
                         sot.volumes_used)
        self.assertEqual(ABSOLUTE_LIMITS["totalBackupsUsed"],
                         sot.backups_used)
        self.assertEqual(ABSOLUTE_LIMITS["totalGigabytesUsed"],
                         sot.gigabytes_used)
        


class TestRateLimit(testtools.TestCase):

    def test_basic(self):
        sot = limits.RateLimit()
        self.assertIsNone(sot.resource_key)
        self.assertIsNone(sot.resources_key)
        self.assertEqual("", sot.base_path)
        self.assertIsNone(sot.service)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_list)

    def test_make_it(self):
        sot = limits.RateLimit(**RATE_LIMIT)
        self.assertEqual(RATE_LIMIT["regex"], sot.regex)
        self.assertEqual(RATE_LIMIT["uri"], sot.uri)
        self.assertEqual(RATE_LIMIT["limit"], sot.limits)


class TestLimits(testtools.TestCase):

    def test_basic(self):
        sot = limits.Limits()
        self.assertEqual("limits", sot.resource_key)
        self.assertEqual("/limits", sot.base_path)
        self.assertEqual("compute", sot.service.service_type)
        self.assertTrue(sot.allow_get)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_list)

    def test_get(self):
        sess = mock.Mock()
        resp = mock.Mock()
        sess.get.return_value = resp
        resp.json.return_value = LIMITS_BODY

        sot = limits.Limits().get(sess)

        self.assertEqual(ABSOLUTE_LIMITS["maxImageMeta"],
                         sot.absolute.image_meta)
        self.assertEqual(ABSOLUTE_LIMITS["maxPersonality"],
                         sot.absolute.personality)
        self.assertEqual(ABSOLUTE_LIMITS["maxPersonalitySize"],
                         sot.absolute.personality_size)
        self.assertEqual(ABSOLUTE_LIMITS["maxSecurityGroupRules"],
                         sot.absolute.security_group_rules)
        self.assertEqual(ABSOLUTE_LIMITS["maxSecurityGroups"],
                         sot.absolute.security_groups)
        self.assertEqual(ABSOLUTE_LIMITS["maxServerMeta"],
                         sot.absolute.server_meta)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalCores"],
                         sot.absolute.cores)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalFloatingIps"],
                         sot.absolute.floating_ips)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalInstances"],
                         sot.absolute.instances)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalKeypairs"],
                         sot.absolute.keypairs)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalRAMSize"],
                         sot.absolute.ram)
        self.assertEqual(ABSOLUTE_LIMITS["maxServerGroups"],
                         sot.absolute.server_groups)
        self.assertEqual(ABSOLUTE_LIMITS["maxServerGroupMembers"],
                         sot.absolute.server_group_members)
        self.assertEqual(ABSOLUTE_LIMITS["totalFloatingIpsUsed"],
                         sot.absolute.floating_ips_used)
        self.assertEqual(ABSOLUTE_LIMITS["totalSecurityGroupsUsed"],
                         sot.absolute.security_groups_used)
        self.assertEqual(ABSOLUTE_LIMITS["totalRAMUsed"],
                         sot.absolute.ram_used)
        self.assertEqual(ABSOLUTE_LIMITS["totalInstancesUsed"],
                         sot.absolute.instances_used)
        self.assertEqual(ABSOLUTE_LIMITS["totalServerGroupsUsed"],
                         sot.absolute.server_groups_used)
        self.assertEqual(ABSOLUTE_LIMITS["totalCoresUsed"],
                         sot.absolute.cores_used)

        self.assertEqual(ABSOLUTE_LIMITS["totalSnapshotsUsed"],
                         sot.absolute.snapshots_used)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalBackups"],
                         sot.absolute.backups)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalVolumeGigabytes"],
                         sot.absolute.volume_gigabytes)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalSnapshots"],
                         sot.absolute.snapshots)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalBackupGigabytes"],
                         sot.absolute.backup_gigabytes)
        self.assertEqual(ABSOLUTE_LIMITS["totalBackupGigabytesUsed"],
                         sot.absolute.backup_gigabytes_used)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalVolumes"],
                         sot.absolute.volumes)
        self.assertEqual(ABSOLUTE_LIMITS["totalVolumesUsed"],
                         sot.absolute.volumes_used)
        self.assertEqual(ABSOLUTE_LIMITS["totalBackupsUsed"],
                         sot.absolute.backups_used)
        self.assertEqual(ABSOLUTE_LIMITS["totalGigabytesUsed"],
                         sot.absolute.gigabytes_used)

        self.assertEqual(RATE_LIMIT["uri"], sot.rate[0].uri)
        self.assertEqual(RATE_LIMIT["regex"], sot.rate[0].regex)
        self.assertEqual(RATE_LIMIT["limit"], sot.rate[0].limits)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.sss import sss_service
from ecl import resource2

class Role(resource2.Resource):
    resource_key = None
    resources_key = None
    base_path = '/roles'
    service = sss_service.SssService()

    # Capabilities
    allow_create = True
    allow_delete = True
    put_create = False

    # Properties
    #: User's id who was attached with the role.
    user_id = resource2.Body('user_id')
    #: Region id which the tenant belongs to.
    region_id = resource2.Body('region_id')
    #: Region name which the tenant belongs to.
    region_name = resource2.Body('region_name')
    #: Tenant's id which is attached with the role.
    tenant_id = resource2.Body('tenant_id')
    #: Tenant's name which is attached with the role.
    tenant_name = resource2.Body('tenant_name')
    #: Attached role's name.
    role_name = resource2.Body('role_name', alternate_id=True)

    def delete(self, session, tenant_id, user_id):
        """Delete role with given tenant id and user id"""

        url = '/roles/tenants/%s/users/%s' % (tenant_id, user_id)
        resp = session.delete(url, endpoint_filter=self.service)
        self._translate_response(resp, has_body=False)
        return self






# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.baremetal import baremetal_service
from ecl import exceptions
from ecl import resource2


class Server(resource2.Resource):
    resources_key = "servers"
    resource_key = "server"
    base_path = '/servers'
    service = baremetal_service.BaremetalService()

    # Capabilities
    allow_get = True
    allow_list = True
    allow_create = True
    allow_delete = True

    # Properties
    #: UUID of the Baremetal server.
    id = resource2.Body('id')
    #: Link of the Baremetal server.
    links = resource2.Body('links')
    #: Name of the Baremetal server.
    name = resource2.Body('name')
    #: Password for the administrator.
    #: This parameter is shown only once, so you have to memo.
    #: Please refer OS specific limitations adminPass raw to know the
    #: required administrator name for each OS.
    adminPass = resource2.Body('adminPass')
    admin_pass = resource2.Body('adminPass')
    #: SSH Keypair name you created on KeyPairs API.
    key_name = resource2.Body('key_name')
    #: Power state: {"RUNNING":"The server is running.",
    #: "SHUTDOWN":"The server was shut down.",
    #: "CRASHED":"The server lost powor controll information temporary.
    #: Status transfer RUNNING/SHUTDOWN Automatically."}
    OS_EXT_STS_power_state = resource2.Body('OS-EXT-STS:power_state')
    power_state = resource2.Body('OS-EXT-STS:power_state')
    #: Task state: {None:"There are no tasks. Only this status can be
    #: received API requests.", "BUILDING":"The task is building the
    #: Baremetal Server. This status is transfered by Create Server.",
    #: "DELETING":"The task is deleting the Baremetal Server. This status
    #: is transfered by Delete Server.", "STOPPING":"The task is stopping
    #: the Baremetal Server. This status is transfered by Stop Server.",
    #: "STARTING":"The task is starting the Baremetal Server. This status
    #: is transfered by Start Server.", "REBOOTING":"The task is rebooting
    #: the Baremetal Server. This status is transfered by Reboot Server."}
    OS_EXT_STS_task_state = resource2.Body('OS-EXT-STS:task_state')
    task_state = resource2.Body('OS-EXT-STS:task_state')
    #: VM state: {"ACTIVE":"The Baremetal server is active.
    #: This status is target of billing.", "BUILD":"The Baremetal server
    #: is built.", "ERROR": "The Baremetal server is outputing a error.
    #: There is necessity to report to our support.", "DELETED":"The
    #: Baremetal server is deleted."}
    OS_EXT_STS_vm_state = resource2.Body('OS-EXT-STS:vm_state')
    vm_state = resource2.Body('OS-EXT-STS:vm_state')
    #: Server's availability zone.
    OS_EXT_AZ_availability_zone = resource2.Body('OS-EXT-AZ:availability_zone')
    availability_zone = resource2.Body('OS-EXT-AZ:availability_zone')
    #: Sever's created time.
    created = resource2.Body('created')
    #: Server's flavor information.
    flavor = resource2.Body('flavor')
    #: Server's host ID.
    hostId = resource2.Body('hostId')
    #: Server's ID.
    image = resource2.Body('image')
    #: Server's image information.
    metadata = resource2.Body('metadata')
    #: Server's links information.
    progress = resource2.Body('progress')
    #: Server's metadata information.
    status = resource2.Body('status')
    #: Server's tenant id.
    tenant_id = resource2.Body('tenant_id')
    #: Server's updated time.
    updated = resource2.Body('updated')
    #: Server's user id.
    user_id = resource2.Body('user_id')
    #: Server's raid arrays information.
    raid_arrays = resource2.Body('raid_arrays')
    #: Server's lvm volume groups information.
    lvm_volume_groups = resource2.Body('lvm_volume_groups')
    #: Server's file system information.
    filesystems = resource2.Body('filesystems')
    #: Server's nic physical ports information.
    nic_physical_ports = resource2.Body('nic_physical_ports')
    #: Server's chassis status information.
    chassis_status = resource2.Body('chassis-status')
    #: managed_by_service
    managed_by_service = resource2.Body('managed_by_service')
    media_attachments = resource2.Body('media_attachments')
    #: key_pair name
    key_name = resource2.Body('key_name')

    def create(self, session, **attrs):
        body = {"server": attrs}
        resp = session.post(
            self.base_path, endpoint_filter=self.service,
            json=body,
            headers={"Accept": "application/json"}
        )
        self._translate_response(resp, has_body=True)
        return self


class ServerDetail(Server):
    base_path = '/servers/detail'

    # capabilities
    allow_create = False
    allow_get = False
    allow_update = False
    allow_delete = False
    allow_list = True



class ServerAction(resource2.Resource):
    resource_key = "console"
    resources_key = None
    base_path = '/servers/%s/action'
    service = baremetal_service.BaremetalService()

    # Properties
    #: Type of the remote console. Valid values are IPMI or IMM.
    type = resource2.Body('type')
    #: URL to access the remote console.
    url = resource2.Body('url')
    #: User ID for sign in to the remote console.
    user_id = resource2.Body('user_id')
    #: Password for sign in to the remote console.
    password = resource2.Body('password')

    def start(self, session, server_id, boot_mode):
        uri = self.base_path % server_id
        body = {"os-start": {"boot_mode": boot_mode}}
        resp = session.post(
            uri,
            endpoint_filter=self.service,
            json=body
        )
        self._translate_response(resp, has_body=False)
        return self

    def stop(self, session, server_id, type):
        uri = self.base_path % server_id
        if type is None:
            body = {"os-stop": None}
        else:
            body = {"os-stop": {"type": type}}
        resp = session.post(
            uri,
            endpoint_filter=self.service,
            json=body
        )
        self._translate_response(resp, has_body=False)
        return self

    def reboot(self, session, server_id, type, boot_mode):
        uri = self.base_path % server_id
        body = {
            "reboot": {
                "type": type,
                "boot_mode": boot_mode,
            }
        }
        resp = session.post(
            uri,
            endpoint_filter=self.service,
            json=body
        )
        self._translate_response(resp, has_body=False)
        return self

    def media_attach(self, session, server_id, image):
        uri = self.base_path % server_id
        body = {
            "media-attach": {
                "imageRef": image
            }
        }
        resp = session.post(
            uri,
            endpoint_filter=self.service,
            json=body
        )
        self._translate_response(resp, has_body=False)
        return self

    def media_detach(self, session, server_id, image):
        uri = self.base_path % server_id
        body = {
            "media-detach": {
                "imageRef": image
            }
        }
        resp = session.post(
            uri,
            endpoint_filter=self.service,
            json=body
        )
        self._translate_response(resp, has_body=False)
        return self

    def get_management_console(self, session, server_id):
        uri = self.base_path % server_id
        body = {
            "os-getManagementConsole": None
        }
        resp = session.post(
            uri,
            endpoint_filter=self.service,
            json=body,
            headers={"Accept": "application/json"}
        )
        self._translate_response(resp, has_body=True)
        return self

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=False, **params):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
        :type session: :class:`~ecl.session.Session`
        :param name_or_id: This resource's identifier, if needed by
                           the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
                            underlying methods, such as to
                            :meth:`~ecl.resource2.Resource.existing`
                            in order to pass on URI parameters.

        :return: The :class:`Resource` object matching the given name or id
                 or None if nothing matches.
        :raises: :class:`ecl.exceptions.DuplicateResource` if more
                 than one resource is found for this request.
        :raises: :class:`ecl.exceptions.ResourceNotFound` if nothing
                 is found and ignore_missing is ``False``.
        """
        # Try to short-circuit by looking directly for a matching ID.

        data = cls.list(session, **params)

        result = cls._get_one_match(name_or_id, data)
        if result is not None:
            return result

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base

class TestTenant(base.BaseFunctionalTest):

    @classmethod
    def test_01_create_tenant(cls):
        tenant = cls.conn.sss.create_tenant(
            tenant_name="T01test",
            description="forSDKtest",
            region="lab3ec",
            #contract_id="econ123467890",
        )
        cls.tenant_id = tenant.id
        print tenant.id
        assert isinstance(tenant.id, six.string_types)
        assert isinstance(tenant.tenant_name, six.string_types)
        assert isinstance(tenant.description, six.string_types)
        assert isinstance(tenant.region, six.string_types)
        assert isinstance(tenant.contract_id, six.string_types)

    def test_02_delete_tenant(self):
        self.conn.sss.delete_tenant(self.tenant_id)




# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy

import testtools

from ecl.block_store.v2 import volume

FAKE_ID = "6685584b-1eac-4da6-b5c3-555430cf68ff"

VOLUME = {
    "status": "creating",
    "name": "my_volume",
    "attachments": [],
    "availability_zone": "nova",
    "bootable": "false",
    "created_at": "2015-03-09T12:14:57.233772",
    "description": "something",
    "volume_type": "some_type",
    "snapshot_id": "93c2e2aa-7744-4fd6-a31a-80c4726b08d7",
    "source_volid": None,
    "imageRef": "some_image",
    "metadata": {},
    "id": FAKE_ID,
    "size": 10
}

DETAILS = {
    "os-vol-host-attr:host": "127.0.0.1",
    "os-vol-tenant-attr:tenant_id": "some tenant",
    "os-vol-mig-status-attr:migstat": "done",
    "os-vol-mig-status-attr:name_id": "93c2e2aa-7744-4fd6-a31a-80c4726b08d7",
    "replication_status": "nah",
    "os-volume-replication:extended_status": "really nah",
    "consistencygroup_id": "123asf-asdf123",
    "os-volume-replication:driver_data": "ahasadfasdfasdfasdfsdf",
    "snapshot_id": "93c2e2aa-7744-4fd6-a31a-80c4726b08d7",
    "encrypted": "false",
}

VOLUME_DETAIL = copy.copy(VOLUME)
VOLUME_DETAIL.update(DETAILS)


class TestVolume(testtools.TestCase):

    def test_basic(self):
        sot = volume.Volume(VOLUME)
        self.assertEqual("volume", sot.resource_key)
        self.assertEqual("volumes", sot.resources_key)
        self.assertEqual("id", sot.id_attribute)
        self.assertEqual("/volumes", sot.base_path)
        self.assertEqual("volumev2", sot.service.service_type)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_retrieve)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_create(self):
        sot = volume.Volume(VOLUME)
        self.assertEqual(VOLUME["id"], sot.id)
        self.assertEqual(VOLUME["status"], sot.status)
        self.assertEqual(VOLUME["attachments"], sot.attachments)
        self.assertEqual(VOLUME["availability_zone"], sot.availability_zone)
        self.assertFalse(sot.is_bootable)
        self.assertEqual(VOLUME["created_at"], sot.created_at)
        self.assertEqual(VOLUME["description"], sot.description)
        self.assertEqual(VOLUME["volume_type"], sot.volume_type)
        self.assertEqual(VOLUME["snapshot_id"], sot.snapshot_id)
        self.assertEqual(VOLUME["source_volid"], sot.source_volume_id)
        self.assertEqual(VOLUME["metadata"], sot.metadata)
        self.assertEqual(VOLUME["size"], sot.size)
        self.assertEqual(VOLUME["imageRef"], sot.image_id)


class TestVolumeDetail(testtools.TestCase):

    def test_basic(self):
        sot = volume.VolumeDetail(VOLUME_DETAIL)
        self.assertIsInstance(sot, volume.Volume)
        self.assertEqual("/volumes/detail", sot.base_path)

    def test_create(self):
        sot = volume.VolumeDetail(VOLUME_DETAIL)
        self.assertEqual(VOLUME_DETAIL["os-vol-host-attr:host"], sot.host)
        self.assertEqual(VOLUME_DETAIL["os-vol-tenant-attr:tenant_id"],
                         sot.project_id)
        self.assertEqual(VOLUME_DETAIL["os-vol-mig-status-attr:migstat"],
                         sot.migration_status)
        self.assertEqual(VOLUME_DETAIL["os-vol-mig-status-attr:name_id"],
                         sot.migration_id)
        self.assertEqual(VOLUME_DETAIL["replication_status"],
                         sot.replication_status)
        self.assertEqual(
            VOLUME_DETAIL["os-volume-replication:extended_status"],
            sot.extended_replication_status)
        self.assertEqual(VOLUME_DETAIL["consistencygroup_id"],
                         sot.consistency_group_id)
        self.assertEqual(VOLUME_DETAIL["os-volume-replication:driver_data"],
                         sot.replication_driver_data)
        self.assertFalse(sot.is_encrypted)



# -*- coding: utf-8 -*-

from ecl.tests.functional import base
from ecl.network.v2.publicip import PublicIP


class TestPublic(base.BaseFunctionalTest):
    @classmethod
    def retrieve_gw(cls):
        cls.gw_name = "Test-sdk-inet-gw"
        inet_gws = cls.conn.network.internet_gateways()
        for inet_gw in inet_gws:
            if inet_gw.name == cls.gw_name:
                cls.one_inet_gw = inet_gw
                break

    @classmethod
    def setUpClass(cls):
        super(TestPublic, cls).setUpClass()
        cls.one_publicip = None
        cls.pip_name = "Test-sdk-publicip"
        cls.one_inet_gw = None
        cls.retrieve_gw()
        publicips = cls.conn.network.publicips()
        for publicip in publicips:
            if publicip.internet_gw_id == cls.one_inet_gw.id:
                cls.one_publicip = publicip
                break
        if not cls.one_publicip:
            cls.one_publicip = cls.conn.network.create_publicip(
                cls.one_inet_gw.id, 28, name=cls.pip_name)
            cls.one_publicip.wait_for_publicip(cls.conn.network.session)

    def _test_get(self):
        sot = self.conn.network.get_publicip(self.one_publicip.id)
        self.assertEqual(self.one_publicip.id, sot.id)

    def _test_delete(self):
        sot = self.conn.network.delete_publicip(self.one_publicip.id)
        self.assertIsInstance(sot, PublicIP)

    def test_update(self):
        if self.one_publicip:
            sot = self.conn.network.update_public_ip(self.one_publicip.id,
                                                          description="Test SDK")
            self.assertIsInstance(sot, PublicIP)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import testtools

from ecl.object_store.v1 import container


CONTAINER_NAME = "mycontainer"

CONT_EXAMPLE = {
    "count": 999,
    "bytes": 12345,
    "name": CONTAINER_NAME
}

HEAD_EXAMPLE = {
    'content-length': '346',
    'x-container-object-count': '2',
    'accept-ranges': 'bytes',
    'id': 'tx1878fdc50f9b4978a3fdc-0053c31462',
    'date': 'Sun, 13 Jul 2014 23:21:06 GMT',
    'x-container-read': 'read-settings',
    'x-container-write': 'write-settings',
    'x-container-sync-to': 'sync-to',
    'x-container-sync-key': 'sync-key',
    'x-container-bytes-used': '630666',
    'x-versions-location': 'versions-location',
    'content-type': 'application/json; charset=utf-8',
    'x-timestamp': '1453414055.48672'
}

LIST_EXAMPLE = [
    {
        "count": 999,
        "bytes": 12345,
        "name": "container1"
    },
    {
        "count": 888,
        "bytes": 54321,
        "name": "container2"
    }
]


class TestContainer(testtools.TestCase):

    def setUp(self):
        super(TestContainer, self).setUp()
        self.resp = mock.Mock()
        self.resp.body = {}
        self.resp.json = mock.Mock(return_value=self.resp.body)
        self.resp.headers = {"X-Trans-Id": "abcdef"}
        self.sess = mock.Mock()
        self.sess.put = mock.Mock(return_value=self.resp)
        self.sess.post = mock.Mock(return_value=self.resp)

    def test_basic(self):
        sot = container.Container.new(**CONT_EXAMPLE)
        self.assertIsNone(sot.resources_key)
        self.assertEqual('name', sot.id_attribute)
        self.assertEqual('/', sot.base_path)
        self.assertEqual('object-store', sot.service.service_type)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_retrieve)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_head)

    def test_make_it(self):
        sot = container.Container.new(**CONT_EXAMPLE)
        self.assertEqual(CONT_EXAMPLE['name'], sot.id)
        self.assertEqual(CONT_EXAMPLE['name'], sot.name)
        self.assertEqual(CONT_EXAMPLE['count'], sot.count)
        self.assertEqual(CONT_EXAMPLE['bytes'], sot.bytes)

    def test_create_and_head(self):
        sot = container.Container(CONT_EXAMPLE)

        # Update container with HEAD data
        sot._attrs.update({'headers': HEAD_EXAMPLE})

        # Attributes from create
        self.assertEqual(CONT_EXAMPLE['name'], sot.id)
        self.assertEqual(CONT_EXAMPLE['name'], sot.name)
        self.assertEqual(CONT_EXAMPLE['count'], sot.count)
        self.assertEqual(CONT_EXAMPLE['bytes'], sot.bytes)

        # Attributes from header
        self.assertEqual(int(HEAD_EXAMPLE['x-container-object-count']),
                         sot.object_count)
        self.assertEqual(int(HEAD_EXAMPLE['x-container-bytes-used']),
                         sot.bytes_used)
        self.assertEqual(HEAD_EXAMPLE['x-container-read'],
                         sot.read_ACL)
        self.assertEqual(HEAD_EXAMPLE['x-container-write'],
                         sot.write_ACL)
        self.assertEqual(HEAD_EXAMPLE['x-container-sync-to'],
                         sot.sync_to)
        self.assertEqual(HEAD_EXAMPLE['x-container-sync-key'],
                         sot.sync_key)
        self.assertEqual(HEAD_EXAMPLE['x-versions-location'],
                         sot.versions_location)
        self.assertEqual(HEAD_EXAMPLE['x-timestamp'], sot.timestamp)

    @mock.patch("ecl.resource.Resource.list")
    def test_list(self, fake_list):
        fake_val = [container.Container.existing(**ex) for ex in LIST_EXAMPLE]
        fake_list.return_value = fake_val

        # Since the list method is mocked out, just pass None for the session.
        response = container.Container.list(None)

        self.assertEqual(len(LIST_EXAMPLE), len(response))
        for item in range(len(response)):
            self.assertEqual(container.Container, type(response[item]))
            self.assertEqual(LIST_EXAMPLE[item]["name"], response[item].name)
            self.assertEqual(LIST_EXAMPLE[item]["count"], response[item].count)
            self.assertEqual(LIST_EXAMPLE[item]["bytes"], response[item].bytes)

    def _test_create_update(self, sot, sot_call, sess_method):
        sot.read_ACL = "some ACL"
        sot.write_ACL = "another ACL"
        sot.is_content_type_detected = True
        headers = {
            "x-container-read": "some ACL",
            "x-container-write": "another ACL",
            "x-detect-content-type": True,
            "Accept": "",
        }
        sot_call(self.sess)

        url = "/%s" % CONTAINER_NAME
        sess_method.assert_called_with(url, endpoint_filter=sot.service,
                                       headers=headers)

    def test_create(self):
        sot = container.Container.new(name=CONTAINER_NAME)
        self._test_create_update(sot, sot.create, self.sess.put)

    def test_update(self):
        sot = container.Container.new(name=CONTAINER_NAME)
        self._test_create_update(sot, sot.update, self.sess.post)

    def _test_no_headers(self, sot, sot_call, sess_method):
        sot = container.Container.new(name=CONTAINER_NAME)
        sot.create(self.sess)
        url = "/%s" % CONTAINER_NAME
        headers = {'Accept': ''}
        self.sess.put.assert_called_with(url, endpoint_filter=sot.service,
                                         headers=headers)

    def test_create_no_headers(self):
        sot = container.Container.new(name=CONTAINER_NAME)
        self._test_no_headers(sot, sot.create, self.sess.put)

    def test_update_no_headers(self):
        sot = container.Container.new(name=CONTAINER_NAME)
        self._test_no_headers(sot, sot.update, self.sess.post)



# -*- coding: utf-8 -*-

from ecl.security_order import security_order_service
from ecl import resource2
from ecl import exceptions
from ecl import utils


class HADevice(resource2.Resource):
    resource_key = None
    resources_key = None
    base_path = '/API/SoEntryFGHA'
    service = security_order_service.SecurityOrderService()

    # Capabilities
    allow_create = True
    allow_get = True
    allow_delete = True
    allow_list = True
    allow_update = True

    # Properties
    #: Tenant ID of the owner (UUID).
    tenant_id = resource2.Body('tenant_id')
    #: List of following objects.
    #: operatingmode: Set "FW" or "UTM" to this value.
    #: licensekind: Set "02" or "08" as FW/UTM plan.
    #: azgroup: Availability Zone.
    gt_host = resource2.Body('gt_host')
    #: A: Create Single Constitution Device.
    #: M: Update Single Constitution Device.
    #: D: Delete Single Constitution Device.
    sokind = resource2.Body('sokind')
    #: Messages are displayed in Japanese or English depending on this value.
    #: ja: Japanese, en: English. Default value is "en".
    locale = resource2.Body('locale')
    #: This value indicates normal or abnormal. 1:normal, 2:abnormal.
    code = resource2.Body('code', alternate_id=True)
    #: This message is shown when error has occurred.
    message = resource2.Body('message')
    #: Identification ID of Service Order.
    soid = resource2.Body('soId')
    #: This value indicates normal or abnormal. 1:normal, 2:abnormal.
    status = resource2.Body('status')
    #: Number of devices.
    records = resource2.Body('records')
    #: Device list.
    rows = resource2.Body('rows')
    #: List of device objects.
    devices = resource2.Body('devices')
    #: Percentage of Service Order Progress Status.
    progress_rate = resource2.Body('progressRate')

    def update(self, session, **body):
        uri = self.base_path
        resp = session.post(uri, endpoint_filter=self.service, json=body)
        self._translate_response(resp, has_body=True)
        return self

    def delete(self, session, body, locale=None):
        uri = self.base_path
        resp = session.post(uri, endpoint_filter=self.service, json=body)
        self._translate_response(resp, has_body=True)
        return self

    def list(self, session, locale=None):
        tenant_id = session.get_project_id()
        uri = '/API/ScreenEventFGHADeviceGet?tenant_id=%s' % tenant_id
        if locale is not None:
            uri += '&locale=%s' % locale
        headers = {'Content-Type': 'application/json'}
        resp = session.get(uri, endpoint_filter=self.service, headers=headers)
        body = resp.json()
        devices = []
        for row in body['rows']:
            device = {
                'internal_use': row['cell'][0],
                'rows': row['cell'][1],
                'ha_id': row['cell'][2],
                'hostname': row['cell'][3],
                'menu': row['cell'][4],
                'plan': row['cell'][5],
                'redundancy': row['cell'][6],
                'availability_zone': row['cell'][7],
                'zone_name': row['cell'][8],
                'halink1networkid': row['cell'][9],
                'halink1subnetid': row['cell'][10],
                'halink1ipaddress': row['cell'][11],
                'halink2networkid': row['cell'][12],
                'halink2subnetid': row['cell'][13],
                'halink2ipaddress': row['cell'][14],
            }
            devices.append(device)
        body.update({'devices': devices})
        self._translate_list_response(resp, body, has_body=True)
        return self

    def _translate_list_response(self, response, body, has_body=True):
        if has_body:
            if self.resource_key and self.resource_key in body:
                body = body[self.resource_key]

            body = self._filter_component(body, self._body_mapping())
            self._body.attributes.update(body)
            self._body.clean()

        headers = self._filter_component(response.headers,
                                         self._header_mapping())
        self._header.attributes.update(headers)
        self._header.clean()



# -*- coding: utf-8 -*-


import uuid

from ecl.network.v2 import network
from ecl.tests.functional import base


class TestNetwork(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestNetwork, cls).setUpClass()
        cls.one_network = None
        nws = cls.conn.network.networks()
        for nw in nws:
            if nw.name == "test-network":
                cls.one_network = nw
                break

    def test_find(self):
        sot = self.conn.network.find_network(self.one_network.name)
        self.assertEqual(self.one_network.id, sot.id)

    def test_get(self):
        sot = self.conn.network.get_network(self.one_network.id)
        self.assertEqual(self.one_network.id, sot.id)
        self.assertEqual(self.one_network.name, sot.name)
        self.assertEqual(self.one_network.description, sot.description)

    def test_list(self):
        names = [o.name for o in self.conn.network.networks()]
        self.assertIn(self.one_network.name, names)

    def test_create(self):
        test_network = self.conn.network.create_network(
            name="test-network"
        )
        self.assertEqual(test_network.name, "test-network")

    def test_delete(self):
        sot = self.conn.network.find_network("test-network")
        self.conn.network.delete_network(sot.id)

    def test_update(self):
        test_description = "test edit the network(edit)"
        sot = self.conn.network.update_network(
            self.one_network.id,
            description=test_description
        )
        self.assertEqual(test_description, sot.description)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.dedicated_hypervisor import dedicated_hypervisor_service
from ecl import resource2


class License(resource2.Resource):
    resources_key = "licenses"
    resource_key = "license"
    base_path = '/licenses'
    service = dedicated_hypervisor_service.DedicatedHypervisorService()

    # Capabilities
    allow_list = True
    allow_create = True
    allow_delete = True

    _query_mapping = resource2.QueryParameters("license_type")

    # Properties
    #: id of Guest Image license.
    id = resource2.Body('id')
    #: license key.
    key = resource2.Body('key')
    #: date the license assigned from.
    assigned_from = resource2.Body('assigned_from')
    #: expiration date for the license.
    expires_at = resource2.Body('expires_at')
    #: license_type of the license
    license_type = resource2.Body('license_type')

    def create(self, session, l_type):
        url = self.base_path
        resp = session.post(url, endpoint_filter = self.service,
                            json={"license_type": l_type})
        self._translate_response(resp, has_body=True)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.network.v2 import port

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    "admin_state_up": True,
    "allowed_address_pairs": [],
    "description": "Example port 1 description.",
    "device_id": "IDENTIFIER",
    "device_owner": "compute:test-device1",
    "fixed_ips": [
      {
        "ip_address": "10.10.10.10",
        "subnet_id": IDENTIFIER
      }
    ],
    "mac_address": "test-mac",
    "name": "Example port 1",
    "network_id": IDENTIFIER,
    "segmentation_id": 0,
    "segmentation_type": "flat",
    "tags": {
      "keyword1": "value1",
      "keyword2": "value2"
    },
    "tenant_id": IDENTIFIER,
    "status": "ACTIVE",
}


class TestPort(testtools.TestCase):

    def test_basic(self):
        sot = port.Port()
        self.assertEqual('port', sot.resource_key)
        self.assertEqual('ports', sot.resources_key)
        self.assertEqual('/ports', sot.base_path)
        self.assertEqual('network', sot.service.service_type)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = port.Port(**EXAMPLE)
        self.assertTrue(sot.admin_state_up)
        self.assertEqual('UP', sot.admin_state)
        self.assertEqual(EXAMPLE['allowed_address_pairs'],
                         sot.allowed_address_pairs)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['device_id'], sot.device_id)
        self.assertEqual(EXAMPLE['device_owner'], sot.device_owner)
        self.assertEqual(EXAMPLE['fixed_ips'], sot.fixed_ips)
        self.assertEqual(EXAMPLE['mac_address'], sot.mac_address)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['network_id'], sot.network_id)
        self.assertEqual(EXAMPLE['segmentation_id'], sot.segmentation_id)
        self.assertEqual(EXAMPLE['segmentation_type'], sot.segmentation_type)
        self.assertEqual(EXAMPLE['tags'], sot.tags)
        self.assertEqual(EXAMPLE['tenant_id'], sot.project_id)
        self.assertEqual(EXAMPLE['status'], sot.status)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.block_store.v2 import type

FAKE_ID = "6685584b-1eac-4da6-b5c3-555430cf68ff"
TYPE = {
    "extra_specs": {
        "capabilities": "gpu"
    },
    "id": FAKE_ID,
    "name": "SSD"
}


class TestType(testtools.TestCase):

    def test_basic(self):
        sot = type.Type(TYPE)
        self.assertEqual("volume_type", sot.resource_key)
        self.assertEqual("volume_types", sot.resources_key)
        self.assertEqual("id", sot.id_attribute)
        self.assertEqual("/types", sot.base_path)
        self.assertEqual("volumev2", sot.service.service_type)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_retrieve)
        self.assertFalse(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_new(self):
        sot = type.Type.new(id=FAKE_ID)
        self.assertEqual(FAKE_ID, sot.id)

    def test_create(self):
        sot = type.Type(TYPE)
        self.assertEqual(TYPE["id"], sot.id)
        self.assertEqual(TYPE["extra_specs"], sot.extra_specs)
        self.assertEqual(TYPE["name"], sot.name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl import utils
from ecl.storage import storage_service

from ecl import resource2
from ecl.storage import storage_service


class Snapshot(resource2.Resource):
    resource_key = 'snapshot'
    resources_key = 'snapshots'
    base_path = '/snapshots'

    service = storage_service.StorageService()

    # capabilities
    allow_list = True
    allow_get = True
    allow_create = True
    allow_delete = True
    allow_update = True

    # query mappings
    _query_mapping = resource2.QueryParameters('volume_id',)

    # Properties
    #: id of snapshot(UUID)
    id = resource2.Body('id')
    #: status of volume
    status = resource2.Body('status')
    #: Name of volume
    name = resource2.Body('name')
    #: Description of snapshot
    description = resource2.Body('description')
    #: Usage of snapshot
    usage = resource2.Body('usage', type=int)
    #: ID (UUID) of parent volume
    volume_id = resource2.Body('volume_id')
    #: ID (UUID) of snapshot type
    snapshot_type_id = resource2.Body('snapshot_type_id')
    #: Creation timestamp of snapshot
    created_at = resource2.Body('created_at')
    #: Update timestamp of snapshot
    updated_at = resource2.Body('updated_at')
    #: Delete timestamp of snapshot
    deleted_at = resource2.Body('deleted_at')
    #: Error message for snapshot
    error_message = resource2.Body('error_message')
    #: Reason of automatic deletion
    delete_reason = resource2.Body('delete_reason')

    def _action(self, session, body):
        """Preform server actions given the message body."""
        # NOTE: This is using Server.base_path instead of self.base_path
        # as both Server and ServerDetail instances can be acted on, but
        # the URL used is sans any additional /detail/ part.
        url = utils.urljoin(Snapshot.base_path, self.id, 'action')
        headers = {'Accept': ''}
        return session.post(
            url, endpoint_filter=self.service, json=body, headers=headers)

    def create(self, session, **attrs):
        body = {"snapshot": attrs}
        resp = session.post(
            self.base_path, endpoint_filter=self.service,
            json=body,
            headers={"Accept": "application/json"}
        )
        self._translate_response(resp, has_body=True)
        return self

    def update(self, session, volume_id, has_body=True, **attrs):
        uri = utils.urljoin(self.base_path, volume_id)
        body = {"snapshot": attrs}
        args = {'json': body}
        resp = session.put(uri, endpoint_filter=self.service, **args)
        self._translate_response(resp, has_body)
        return self

    def restore(self, session):
        """Restore volume from specified snapshot"""
        body = {'restore': None}
        self._action(session, body)

class SnapshotDetail(Snapshot):
    base_path = '/snapshots/detail'
    allow_get = False
    allow_create = False
    allow_delete = False
    allow_update = False



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import unittest

from ecl.telemetry.v2 import sample
from ecl.tests.functional import base


@unittest.skipUnless(base.service_exists(service_type="metering"),
                     "Metering service does not exist")
class TestSample(base.BaseFunctionalTest):

    meter = None
    sample = None

    @classmethod
    def setUpClass(cls):
        super(TestSample, cls).setUpClass()
        cls.meter = next(cls.conn.telemetry.meters())
        resource = next(cls.conn.telemetry.resources())
        sot = cls.conn.telemetry.create_sample(
            counter_name=cls.meter.name,
            meter=cls.meter.name,
            counter_type='gauge',
            counter_unit='instance',
            counter_volume=1.0,
            resource_id=resource.id,
        )
        assert isinstance(sot, sample.Sample)
        cls.sample = sot

    def test_list(self):
        ids = [o.id for o in self.conn.telemetry.samples(self.meter)]
        self.assertIn(self.sample.id, ids)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import  testtools
from ecl.sss.v1 import api_key

BASIC_EXAMPLE = {
    "user_id" : "ecid0000011111",
    "keystone_name" : "XTI3u6blgFu36je0jDjkBKlFG4GmI4kn",
    "keystone_password" : "Dbu1nZUprR8wg57k",
    "status" : "Success"
}

class TestRole(testtools.TestCase):

    def test_role(self):
        sot = api_key.Api_key()
        self.assertEqual(None, sot.resources_key)
        self.assertEqual(None, sot.resource_key)
        self.assertEqual("/keys", sot.base_path)
        self.assertEqual('sss', sot.service.service_type)

        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_get)
        self.assertTrue(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_list)

    def test_make_role(self):
        sot = api_key.Api_key(**BASIC_EXAMPLE)
        self.assertEqual(BASIC_EXAMPLE['user_id'], sot.user_id)
        self.assertEqual(BASIC_EXAMPLE['keystone_name'], sot.keystone_name)
        self.assertEqual(BASIC_EXAMPLE['keystone_password'], sot.keystone_password)
        self.assertEqual(BASIC_EXAMPLE['status'], sot.status)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base

class TestFirewallAction(base.BaseFunctionalTest):
    def test_01_reboot_firewall(self):
        reboot = self.conn.network.reboot_firewall(
            "5f29f0f4-ef23-484e-a81f-ea621175f1a3",
            "HARD"
        )

    def test_02_reset_password(self):
        resp = self.conn.network.reset_password_firewall(
            "5f29f0f4-ef23-484e-a81f-ea621175f1a3",
            "user-read"
        )
        print resp
        self.assertIsInstance(resp.new_password, six.string_types)
        self.assertIsInstance(resp.username, six.string_types)


# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import time
import six
from ecl.tests.functional import base

class TestLoadBalancer(base.BaseFunctionalTest):
    def test_09_load_balancers(self):
        load_balancers = list(self.conn.network.load_balancers(
            status="ACTIVE", description="",
            load_balancer_plan_id="29da442c-9a06-4eb3-b3cd-b6b123198214",
            #id="911726aa-a249-4a58-a59b-c233a110649b"
        ))
        print [load_balancer.id for load_balancer in load_balancers]

    @classmethod
    def test_02_create_load_balancer(cls):
        load_balancer = cls.conn.network.create_load_balancer(
            load_balancer_plan_id="29da442c-9a06-4eb3-b3cd-b6b123198214"
        )
        print load_balancer
        cls.id = load_balancer.id
        time.sleep(360)

    def test_03_show_load_balancer(self):
        load_balancer = self.conn.network.show_load_balancer(
            self.id
        )
        print load_balancer
        self.assertIsInstance(load_balancer.admin_username, six.string_types)
        #self.assertIsInstance(load_balancer.availability_zone, six.string_types)
        #self.assertIsInstance(load_balancer.default_gateway, six.string_types)
        #self.assertIsInstance(load_balancer.description, six.string_types)
        self.assertIsInstance(load_balancer.id, six.string_types)
        self.assertIsInstance(load_balancer.interfaces, list)
        self.assertIsInstance(load_balancer.load_balancer_plan_id, six.string_types)
        self.assertIsInstance(load_balancer.name, six.string_types)
        self.assertIsInstance(load_balancer.status, six.string_types)
        self.assertIsInstance(load_balancer.status, six.string_types)
        self.assertIsInstance(load_balancer.tenant_id, six.string_types)
        self.assertIsInstance(load_balancer.user_username, six.string_types)

    def test_04_update_load_balancer(self):
        #self.id = "911726aa-a249-4a58-a59b-c233a110649b"
        load_balancer = self.conn.network.update_load_balancer(
            self.id,
            description="updated"
        )
        print load_balancer

    def test_05_delete_load_balancer(self):
        #self.id = "911726aa-a249-4a58-a59b-c233a110649b"
        self.conn.network.delete_load_balancer(
            self.id
        )


# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.sss import  sss_service
from ecl import resource2


class IAMGroup(resource2.Resource):
    resource_key = None
    resources_key = 'iam_groups'
    base_path = '/iam/groups'

    # Capabilities
    allow_create = True
    allow_delete = True
    allow_list = True
    allow_update = True

    # Properties
    #: The IAM Group ID.
    iam_group_id = resource2.Body('iam_group_id', alternate_id=True)
    #: The IAM Group name.
    iam_group_name = resource2.Body('iam_group_name')
    #: Description of the IAM group.
    description = resource2.Body('description')
    #: IAM Role Object.
    #: * iam_role_id: The IAM Role ID.
    #: * iam_role_name: The IAM Role name.
    iam_roles = resource2.Body('iam_roles')
    #: The IAM Role ID.
    iam_role_id = resource2.Body('iam_role_id')
    #: The IAM Role name.
    iam_role_name = resource2.Body('iam_role_name')
    #: User ID that belongs in the designated IAM Group ID.
    users = resource2.Body('users')
    #: The User ID.
    user_id = resource2.Body('user_id')
    #: The contact ID that the IAM Group belongs.
    contract_id = resource2.Body('contract_id')

    def list(self, session, contract_id):
        """List iam groups by contract id"""

        url = self.base_path + '?contract_id=%s' % contract_id
        resp = session.get(url, endpoint_filter=self.service)
        self._translate_response(resp, has_body=True)
        return self

    def assign_iam_role(self, session, iam_group_id, iam_role_id):
        """Assignment of the IAM Role to the IAM Group."""

        url = self.base_path + '/%s/roles/%s' % (iam_group_id, iam_role_id)
        resp = session.put(url, endpoint_filter=self.service)
        self._translate_response(resp, has_body=True)
        return self

    def delete_assign_iam_role(self, session, iam_group_id, iam_role_id):
        """Delete Assignment of the IAM Role to the IAM Group."""

        url = self.base_path + '/%s/roles/%s' % (iam_group_id, iam_role_id)
        resp = session.delete(url, endpoint_filter=self.service)
        self._translate_response(resp, has_body=False)
        return self

    def list_users(self, session, iam_group_id):
        """Show User list in the designated IAM Group ID."""

        url = self.base_path + '/%s/users' % iam_group_id
        resp = session.get(url, endpoint_filter=self.service)
        self._translate_response(resp, has_body=True)
        return self

    def assign_user(self, session, iam_group_id, user_id):
        """Assignment of the IAM User to the IAM Group."""

        url = self.base_path + '/%s/users/%s' % (iam_group_id, user_id)
        resp = session.put(url, endpoint_filter=self.service)
        self._translate_response(resp, has_body=True)
        return self

    def delete_assign_user(self, session, iam_group_id, user_id):
        """Delete Assignment of the IAM User to the IAM Group."""

        url = self.base_path + '/%s/users/%s' % (iam_group_id, user_id)
        resp = session.delete(url, endpoint_filter=self.service)
        self._translate_response(resp, has_body=False)
        return self
